<?php
/**
 * Файл для работы с автозагрузчиком классов.
 * @package mxl
 * @author merlin <merlin@ersys.ru>
 * @copyright Copyright (c) 2009, ООО НПО Метарекс
 */
/**
 * Класс для работы с автозагрузкой классов.
 * @package mxl
 * @author merlin <merlin@ersys.ru>
 * @copyright Copyright (c) 2009, ООО НПО Метарекс, {@link http://www.metarex.ru www.metarex.ru}
 */
class mxl_autoload {
	/**
	 * Регистрация автозагрузчика
	 */
	public static function register() {
		if (!spl_autoload_register('mxl_autoload::load')) {
			throw new Exception(mxl_cfg_exceptions::$autoload_register);
		}
	}

	/**
	 * Отмена регистрации автозагрузчика
	 */
	public static function unregister() {
		if (!spl_autoload_unregister('mxl_autoload::load')) {
			throw new Exception(mxl_cfg_exceptions::$autoload_unregister);
		}
	}

	/**
	 * Процедура загрузки класса по имени.
	 * @param string $className - имя класса для загрузки
	 */
	public static function load($className) {
		if (strpos($className, MXL_Autoload_pref) !== false){
			$basePath = realpath(dirname(__FILE__)).MXL_Autoload_pathsep;
			$fileToLoad = $basePath . self::getFileByClass($className);

			if (!file_exists($fileToLoad)){
				$errMess = mxl_cfg_exceptions::$autoload_load;
				if (strpos($errMess, '#VTINS') !== false)
					$errMess = str_replace('#VTINS', $fileToLoad, $errMess);
					
				if (strpos($errMess, '#VTINS1') !== false)
					$errMess = str_replace('#VTINS1', $className, $errMess);
				
				throw new Exception($errMess);	
			} else
				@include_once($fileToLoad);
		}
	}

	/**
	 * Метод получения имени фала по имени класса.
	 * @param string $class - имя класса
	 * @return string
	 */
	private static function getFileByClass($class) {
		$class = str_replace(MXL_Autoload_pref, '', $class);
		$fileName = str_replace(MXL_Autoload_bar, MXL_Autoload_pathsep, $class)
				.'.'.MXL_Autoload_ext;
		return $fileName;
	}
}
?>