<?php
/**
 * Файл для описания класса базовых параметров локализации и конфигурации, 
 * {@link mxl_cfg_base}.
 * @package mxl_cfg
 * @author merlin <merlin@ersys.ru>
 * @copyright Copyright (c) 2009, ООО НПО Метарекс
 */
/**
 * Класс описания базовых параметров локализации и конфигурации
 * 
 * @package mxl_cfg
 * @author merlin <merlin@ersys.ru>
 * @copyright Copyright (c) 2009, ООО НПО Метарекс, {@link http://www.metarex.ru www.metarex.ru}
 */
class mxl_cfg_base{
	/**
	 * Расширение json-файлов, предназначенных для загрузки метаданных. 
	 * @var string
	 */
	public static $json_file_ext;
	
	/**
	 * Расширение сниппет-файлов, предназначенных для динамического формирования 
	 * частей json-файлов.
	 * @var string
	 */
	public static $snippet_file_ext;

     /**
     * Временая зона сервера в виде числа из GMT-5. В настоящий момент
     * используется в функциях js сдвига времени.
     */
    public static $server_time_zone;
	
	/**
	 * Разделитель значений при экспорте данных в формат CVS
	 * @var string
	 */
	public static $exportcvs_delimeter;
	
	
	/**
	 * Формат даты локализации по умолчанию.
	 */
	public static $format__date;
	
	/**
	 * Формат времени локализации по умолчанию. 
	 */
	public static $format__time;

    /**
     * Формат даты времени локализации по умолчанию.
     */
    public static $format__date_time;
	
	/**
	 * Метод устанавливает путь к файлам методанных библиотеки.
	 * @param string $path - путь 
	 */
	public static function setMetaXLibcfg_path($path){
		if (!mxl_cfg_path::$separator)
			mxl_cfg_path::$separator = '//';
			
		mxl_cfg_path::$metaxlib_cfg = $path;
	} 

	/**
	 * Метод устанавливает путь к файлам библиотеки 
	 * @param string $path - путь 
	 */
	public static function setMetaXLib_path($path){
		if (!mxl_cfg_path::$separator)
			mxl_cfg_path::$separator = '//';
			
		mxl_cfg_path::$metaxlib = $path;
	} 

	/**
	 * Метод устанавливает url адресс библиотеки
	 * @param string $url
	 */
	public static function setMetaXLib_url($url){
		mxl_cfg_url::$metaxlib = $url;
	}

	/**
	 * Метод читает базовые параметры из файла конфгурации и локализации
	 * @param array $cfg
	 */
	protected static function readJSON($cfg){
		mxl_cfg_base::$json_file_ext       = $cfg['base']['json_file_ext'];
		mxl_cfg_base::$snippet_file_ext    = $cfg['base']['snippet_file_ext'];
		mxl_cfg_base::$server_time_zone    = $cfg['base']['server_time_zone'];
		mxl_cfg_base::$exportcvs_delimeter = $cfg['base']['exportcvs']['delimeter'];
	}

	/**
	 * Метод читает форматы из файла локализации
	 * @param array $cfg
	 */
	protected static function readJSONFormat($cfg){
		mxl_cfg_base::$format__date = $cfg['format']['date'];
		mxl_cfg_base::$format__time = $cfg['format']['time'];
        mxl_cfg_base::$format__date_time = $cfg['format']['date_time'];
	}
	
	/**
	 * Общий метод для чтения всех параметров конфигурации библиотеки
	 * @param string $fileName - имя файла, который необходимо прочитать
	 */
	public static function readCfgFromJSONFile($fileName){
		$fileName = mxl_utils_files::makeAbsolute($fileName);
		$cfg      = mxl_utils_json::readArrayFromFile($fileName, 
				new mxl_core_params());
		
		mxl_cfg_base::readJSON($cfg);
		mxl_cfg_url::readJSON($cfg);
		mxl_cfg_path::readJSON($cfg);
	} 
	/**
	 * 
	 * Общий метод для чтения всех параметров локализации библиотеки
	 * @param string $fileName - имя файла, который необходимо прочитать
	 */
	public static function readLocalizationFromJSONFile($fileName){
		$fileName = mxl_cfg_path::$localizations.mxl_cfg_path::$separator.
				$fileName.'.'.mxl_cfg_base::$json_file_ext;
		$fileName = mxl_utils_files::makeAbsolute($fileName);
		$cfg      = mxl_utils_json::readArrayFromFile($fileName, 
				new mxl_core_params());
		
		mxl_cfg_base::readJSONFormat($cfg);
		mxl_cfg_exceptions::readJSON($cfg);
		mxl_cfg_messages::readJSON($cfg);
	}
}
?>