<?php
/**
 * Файл для описания базового класса метаданного, {@link mxl_core_metadata}.
 * @package mxl_core
 * @author merlin <merlin@ersys.ru>
 * @copyright Copyright (c) 2009, ООО НПО Метарекс
 */
/**
 * Базовый класс метаданного.
 * 
 * @package mxl_core
 * @author merlin <merlin@ersys.ru>
 * @copyright Copyright (c) 2009, ООО НПО Метарекс, {@link http://www.metarex.ru www.metarex.ru}
 */
abstract class mxl_core_metadata{
	/**
	 * Имя метаданного
	 * @var string
	 */
	private $name;
	/**
	 * Метод возвращает имя файла метаданного, предварительно осуществляя проверку
	 * на его существование. Если файл не найден то выбрасывается исключение
	 * {@link mxl_cfg_exceptions::$core_metadata__json_file_not_find}
	 * @param string $directory - директорий сайта
	 * @param string $metaDataName - имя метаданного для загрузки
	 * @exception {@link mxl_cfg_exceptions::$core_metadata__json_file_not_find}
	 * @return false|string
	 */
	public static function getJSONFileName($directory, $metaDataName){
		$fileName = $directory.mxl_cfg_path::$separator
				.$metaDataName.'.'.mxl_cfg_base::$json_file_ext;
		$fileName_rp = realpath($fileName);
		
		if (!file_exists($fileName_rp)){
			$errMess = mxl_cfg_exceptions::$core_metadata__json_file_not_find;
			if (strpos($errMess, '#VTINS') !== false)
				$errMess = str_replace('#VTINS', $fileName,	$errMess);
			
			throw new Exception($errMess);	
			return false;
		}
		return $fileName_rp;
	}

	/**
	 * Абстрактный метод для загрузки данных из json-файла. Необходимо реализовать
	 * в классах наследниках. 
	 * @param String $metaDataName - имя метаданного для загрузки
	 * @param mxl_core_params $params - динамические параметры
	 * @return unknown_type
	 */
	abstract public static function loadFromJSON($metaDataName, mxl_core_params $params);
	/**
	 * Конструктор класса матаданного.
	 * @param string $name - имя метаданного
	 * @return mxl_core_enum
	 */
	function __construct($name){
		$this->name = $name;
	}
	
	/**
	 * Возвращает имя метаданного.
	 * @return string
	 */
	public function getName(){
		return $this->name;
	}
}
?>