<?php
/**
 * Файл для описания класса хранилища метаданных, {@link mxl_core_metadatas}.
 * @package mxl_core
 * @author merlin <merlin@ersys.ru>
 * @copyright Copyright (c) 2009, ООО НПО Метарекс
 */
/**
 * Класс описания хранилища метаданных.
 * 
 * @package mxl_core
 * @author merlin <merlin@ersys.ru>
 * @copyright Copyright (c) 2009, ООО НПО Метарекс, {@link http://www.metarex.ru www.metarex.ru}
 */
class mxl_core_metadatas{
	/**
	 * Статическая переменная для реализации паттерна singleton
	 * @var mxl_core_metadatas
	 */
	private static $metadatas;
	
	/**
	 * Функция реализующая паттерн singleton
	 * @return mxl_core_metadatas
	 */
	public static function instance(){
		if (!isset(mxl_core_metadatas::$metadatas)){
			mxl_core_metadatas::$metadatas = new mxl_core_metadatas();
		}
		return mxl_core_metadatas::$metadatas;
	}
	
	/**
	 * Переменная для идентификации типа перечисления
	 * @var string
	 */
	public static $enum     = 'enum';
	/**
	 * Переменная для идентификации типа кобобокс
	 * @var string
	 */
	public static $combobox = 'combobox';
	/**
	 * Переменная для идентификации типа грид
	 * @var string
	 */
	public static $dbgrid   = 'dbgrid';
	/**
	 * Переменная для идентификации типа список
	 * @var string
	 */
	public static $list = 'list';
	/**
	 * переменная для идентификации типа валидатор
	 * @var string
	 */
	public static $validator = 'validator';
	
	/**
	 * Массив доступных типов метеданных
	 * @var array
	 */
	private $types = array('enum', 'combobox', 'dbgrid', 'list', 'validator');
	/**
	 * Массив для хранения загруженных объектов Метаданых, делится на типы 
	 * метаданных.
	 * @var array
	 */
	private $listMetaData;
	
	/**
	 * Переменная для передачи текущих параметров widget-a 
	 * @var mxl_core_params
	 */
	private $currParams;
	/**
	 * Переменная определяющая пустой набор параметров 
	 * @var mxl_core_params
	 */
	private $emptyParams;

	/**
	 * Конструктор хранилища метаданных.  
	 * @return mxl_core_metadatas
	 */
	public function __construct(){
		$this->listMetaData = array();
		
		foreach ($this->types as $value)
			$this->listMetaData[$value] = array();
			
		$this->currParams  = new mxl_core_params();
		$this->emptyParams = new mxl_core_params();
	}
	
	/**
	 * Деструктор хранилища метаданных. 
	 */
	public function __destruct(){
		foreach ($this->types as $value)
			unset($this->listMetaData[$value]);
			
		unset($this->listMetaData);
		
		unset($this->currParams);
		unset($this->emptyParams);
	}
	
	/**
	 * Функция проверки типа метаданных, в случае если тип не проходит
	 * проверку выбрасывается исключение 
	 * {@link mxl_cfg_exceptions::$core_metadatas_checkType}.
	 * 
	 * @param string $metaDataType - проверяемый тип метаданного
	 * @throws Exception {@link mxl_cfg_exceptions::$core_metadatas_checkType}
	 * @return boolean
	 */
	public function checkType($metaDataType){
		foreach ($this->types as $type)
			if ($type == $metaDataType)
				return true;

		$errMess = mxl_cfg_exceptions::$core_metadatas_checkType;
		if (strpos($errMess, '#VTINS') !== false)
			$errMess = str_replace('#VTINS', $metaDataType, $errMess);
					
		throw new Exception($errMess);
		return false;
	}
	
	/**
	 * Функция проверяе есть ли метаданное с таким типом и именем. Если есть
	 * то выбрасывается исключение 
	 * {@link mxl_cfg_exceptions::$core_metadatas_checkDontHave}.
	 * @param string $metaDataType - тип метаданого
	 * @param string $metaDataName - имя метаданного
	 * @throws Exception {@link mxl_cfg_exceptions::$core_metadatas_checkType},
	 * 		{@link mxl_cfg_exceptions::$core_metadatas_checkDontHave}
	 * @return boolean
	 */
	private function checkDontHave($metaDataType, $metaDataName){
		if (!$this->checkType($metaDataType))
			return false;
		if (array_key_exists($metaDataName, $this->listMetaData[$metaDataType])){
			
			$errMess = mxl_cfg_exceptions::$core_metadatas_checkDontHave;
			if (strpos($errMess, '#VTINS') !== false)
				$errMess = str_replace('#VTINS',  $metaDataName, $errMess);
			if (strpos($errMess, '#VTINS1') !== false)
				$errMess = str_replace('#VTINS1', $metaDataType, $errMess);
				
			throw new Exception($errMess);

			return false;
		} else
			return true; 
	}

	/**
	 * Функция загружает матаданное согласно его типу и динамическим параметрам
	 * @param string $metaDataType - тип метаданного 
	 * @param string $metaDataName - имя метаданного
	 * @param mxl_core_params $params - динамические параметры
	 * @throws Exception {@link mxl_cfg_exceptions::$core_metadatas_checkType}
	 * @return boolean
	 */
	public function loadMetaData($metaDataType, $metaDataName, 
			mxl_core_params $params){
				
		if ($this->checkType($metaDataType)) {
			$metaData = '';
			if ($metaDataType == mxl_core_metadatas::$enum){
				$metaData = mxl_core_enum::loadFromJSON($metaDataName, $params);
			} else if ($metaDataType == mxl_core_metadatas::$combobox){
				$metaData = mxl_dict_combobox::loadFromJSON($metaDataName, $params);
			} else if ($metaDataType == mxl_core_metadatas::$dbgrid){
				$metaData = mxl_dict_dbgrid::loadFromJSON($metaDataName, $params);
			} else if ($metaDataType == mxl_core_metadatas::$validator){
				$metaData = mxl_dict_validator::loadFromJSON($metaDataName, $params);
			} else if ($metaDataType == mxl_core_metadatas::$list){
				$metaData = mxl_dict_list::loadFromJSON($metaDataName, $params);
			} else
				return false;

			if ($this->checkDontHave($metaDataType, $metaData->getName())){
				$this->listMetaData[$metaDataType][$metaDataName] = $metaData;
				return true;
			}
		}
		return false;
	}
	
	/**
	 * Метод для получения метаданного по имени и типу. В случае если метаданное
	 * не удалось найти в хранилище или загрузить из словаря выбрасывается
	 * исключение {@link mxl_cfg_exceptions::$core_metadatas_getMetaData}
	 * @param string $metaDataType - тип метаданного 
	 * @param string $metaDataName - имя метаданного
	 * @throws Exception {@link mxl_cfg_exceptions::$core_metadatas_checkType},
	 * 		{@link mxl_cfg_exceptions::$core_metadatas_getMetaData}
	 * @return mxl_core_metadata
	 */
	public function getMetaData($metaDataType, $metaDataName){
		if ($this->checkType($metaDataType)) {
			if(	(!array_key_exists($metaDataName, $this->listMetaData[$metaDataType]))&&
					(!$this->loadMetaData($metaDataType, $metaDataName,	$this->currParams))){
				
				$errMess = mxl_cfg_exceptions::$core_metadatas_getMetaData;
				if (strpos($errMess, '#VTINS') !== false)
					$errMess = str_replace('#VTINS',  $metaDataName, $errMess);
				if (strpos($errMess, '#VTINS1') !== false)
					$errMess = str_replace('#VTINS1', $metaDataType, $errMess);
				throw new Exception($errMess);	
				return false;
			} 
			return $this->listMetaData[$metaDataType][$metaDataName];
		}
		return false;
	} 
	
	/**
	 * Метод для установки текущего значения динамических параметров.
	 * Используется в виджетах.
	 * @param mxl_core_params $currParams - динамические параметры
	 */
	public function setCurrParams($currParams){
		unset($this->currParams);
		$this->currParams = $currParams;
	} 
	
	/**
	 * Метод для сброса динамических параметров
	 */
	public function setCurrParamsToEmpty(){
		unset($this->currParams);
		$this->currParams = $this->emptyParams;
	} 
	
	/**
	 * Метод для получения текущего значения динамических параметров.
	 * @return mxl_core_params
	 */
	public function getCurrParams(){
		return $this->currParams;
	} 
	
}
?>