<?php
/**
 * Файл для класса описания грида, {@link mxl_dict_dbgrid}.
 * @package mxl_dict
 * @author merlin <merlin@ersys.ru>
 * @copyright Copyright (c) 2009, ООО НПО Метарекс
 */
/**
 * Класс описания грида.
 * Объект можно создать только путем загрузки из json-файла, которая 
 * осуществляется функцией {@link mxl_dict_dbgrid::loadDBGridFromJSON()}.
 * 
 * Структура json-файла:<br>
 * <code>
 * {
 * "name"          : "имя грида",
 * "caption"       : "заголовок грида",
 * "logicBaseName" : "имя логической бд",
 * "tableName"     : "имя базовой таблицы",
 * "primaryKey"    : "имя ключевого поля в базовой таблице",
 * "queryText"     : "запрос с макросами для получения данных",
 * "where"         : "ограничение на отбор данных",
 * "order"         : "порядок сортировки",
 * 
 * 
 * "options" : {
 * 	"readOnly"         : false/true - возможно ли редктировать данные,
 *  "validator"        : "имя файла валидации, расположенного в папке валидации",
 *  "renderer"         : "имя файла скрипта рендерера, расположенного в папке рендереров",
 *  
 *  "nextColOnEnter"   : false/true - режим перехода к редактированию следующего поля строки по enter,
 *  
 * 	"selectByLeftJoin" : false/true - Строить запрос с использованием конструкций left join,
 * 	"manyRowsSaveMode" : false/true - режим сохранения множества строк,
 * 	
 *  "paginMode"        : false/true - режим построничной работы грида, 
 *  "pageStart"        : 0 - задание начальной страницы активно при постраничном режиме работы, 
 *  "pageSize"         : 12 - задание кол-ва строк на странице,
 *  
 * 	"clicksToEdit"     : 1 - число кликов мышки для перехода в режим редактирования,
 * 	"height"           : 405 - высота грида,
 * 	"width"            : 800 - ширина грида
 * },
 * 
 * раздел описания полей
 * "fields" : [
 * 	{"n":"имя поля 1", "c":"Заголовок 1", "w":10 - ширина, "dt":"тип данных", "dl":размер данных, "a":"атрибуты", "mr":"маска редактирования"},
 *   ...
 * 	{"n":"имя поля N", "c":"Заголовок N", "w":25 - ширина, "dt":"тип данных", "dl":размер данных, "a":"атрибуты", "mr":"маска редактирования"},
 * ],
 * раздел описания связей полей
 * "relations" : [
 * пример описания для enum
 * 	{"fn":"имя поля", "rl":"",                                                           "cn":"имя комбобокса"},
 * пример описания для связи с другой таблицой по ключу
 * 	{"fn":"имя поля", 
 * 		"rl":"таблица;
 * 				алиас;
 * 				поле из которого будет барться значение;
 * 				алиас базовой таблицы.ключевое поле в базовой таблице=алиас.клучевое поле в таблице связи",
 *	 "cn":"@имя комбобокса для выбора данных"},
 * 	{"fn":"имя поля", "rl":"алиас таблицы связи;поле из которого будет заполнятся в таблице связи",                                            "cn":""},
 * ],
 * раздел задания значений по умолчанию, для полей со связми к другим 
 * таблицам можно указывать только значение ключевого поля, остальные 
 * значения полей подтянуться автоматически
 * "defaults" : [
 * 	{"fn":"имя поля 1", "df":"значение по умолчанию 1"},
 * ...
 * 	{"fn":"имя поля N", "df":"значение по умолчанию N"}
 * ],
 * 
 * раздел задания итогов
 * "totals" : {
 *  
 *  итоги по гриду
 * 	"grid" : [
 * 		{"fn":"имя поля для итога", "tp":"тип итога", "c":"заголовок итога"},
 * 		...
 * 		{"fn":"имя поля для итога", "tp":"тип итога", "c":"заголовок итога"}
 * 	],
 * 
 * 	итоги в футере
 * 	"footer" : [
 * 		заголовок может принимать сложные занчения для определения как отображать итог:
 *		Значения макроса разделяются '|';
 * 		Макрос для вставки значения #VTINS;
 * 		Возможны три ситуации указания значений:
 * 			1. идентификатор значения не заполнен - общий заголовок
 * 			2. идентифиакфтор #NVAL= - значение определяющие текст отображения для 
 * 				ситуации когда итог не имеет значение.
 * 			3. "значение=" - для указанного значения будет выведен этот текст
 *		{"fn":"имя поля для итога", "tp":"тип итога", "c":"заголовок итога", "h": "текст всплывающего окошка"},
 *		... 
 *		{"fn":"имя поля для итога", "tp":"тип итога", "c":"заголовок итога", "h": "текст всплывающего окошка"} 
 * 	]
 * 
 * }
 * 
 * }
 * 
 * Пример json-файла:
 * {
 * "name"          : "loc1",
 * "caption"       : "Locations",
 * "logicBaseName" : "db_ceifx",
 * "tableName"     : "locations",
 * "primaryKey"    : "location_id",
 * "queryText"     : "",
 * "where"         : "",
 * "order"         : "",
 *  
 * "options" : {
 * 	"readOnly"         : false, "validator" : "loc1",
 * 	"selectByLeftJoin" : false,
 * 	"manyRowsSaveMode" : true,
 * 	"paginMode"        : true, "pageStart" : 0, "pageSize" : 12,
 *  "nextColOnEnter"   : true,
 * 	"clicksToEdit"     : 1,
 * 	"height"           : 405,
 * 	"width"            : 800
 * },
 * "fields" : [
 * 	{"n":"location_id",         "c":"LocationID",          "w":10, "dt":"int",       "dl":0,   "a":"",       "mr":""},
 * 	{"n":"regdate",             "c":"Reg Date",            "w":15, "dt":"edatetime", "dl":40,  "a":"",       "mr":"", "formatD":"D-M-Y", "formatT":"i:H"},
 * 	{"n":"location_name",       "c":"Name",                "w":20, "dt":"string",    "dl":80,  "a":"",       "mr":"//([a-zA-Z0-9\s]+)$//"},
 * 	{"n":"location_name_clean", "c":"Location name clean", "w":10, "dt":"string",    "dl":80,  "a":"|HD|",   "mr":""},
 * 	{"n":"location_type",       "c":"Loc Type",            "w":15, "dt":"string",    "dl":40,  "a":"|ENUM|", "mr":""},
 * 	{"n":"company_id",          "c":"Company ID",          "w":11, "dt":"int",       "dl":0,   "a":"",       "mr":""},
 * 	{"n":"company_name",        "c":"Company",             "w":15, "dt":"string",    "dl":40,  "a":"|NU|",   "mr":""},
 * 	{"n":"company_email",       "c":"Company Email",       "w":25, "dt":"string",    "dl":40,  "a":"|NU|",   "mr":""},
 * ],
 * "relations" : [
 * 	{"fn":"location_type",   "rl":"",                                                           "cn":"combotest"},
 * 	{"fn":"company_name",    "rl":"companies;b;company_display_name;a.company_id=b.company_id", "cn":"@companies", "lw": 200},
 * 	{"fn":"company_email",   "rl":"b;company_email",                                            "cn":""},
 * ],
 * "defaults" : [
 * 	{"fn":"email",         "df":"default@mail.en"},
 * 	{"fn":"city",          "df":"LasVegas"},
 * 	{"fn":"address",       "df":"LasVegas"},
 * 	{"fn":"company_id",    "df": 1},
 * 	{"fn":"location_type", "df": 1}
 * ],
 * "totals" : {
 * 	"grid" : [
 * 		{"fn":"location_id",   "tp":"count", "c":"Locations Count: "},
 * 		{"fn":"company_id",    "tp":"sum",   "c":"Company Summ: "}
 * 	],
 * 	"footer" : [
 * 		{"fn":"location_id",   "tp":"count", "c":"",              "h":""},
 * 		{"fn":"zip",           "tp":"count", "c":"",              "h":"Zip count"},
 * 		{"fn":"location_name", "tp":"count", "c":"Count: #VTINS", "h":"Count of location ids"},
 * 		{"fn":"company_id",    "tp":"sum",   "c":"",              "h":"Company ids summary value"},
 * 		{"fn":"company_name",  "tp":"count", "c":"Companies|#NVAL=No Companies|1=#VTINS Company|12=#VTINS- Comp.",
 * 				"h":"Total no. of companies"}
 * 	]
 * }
 * 
 * }
 * 
 * @package mxl_dict
 * @author merlin <merlin@ersys.ru>
 * @copyright Copyright (c) 2009, ООО НПО Метарекс, {@link http://www.metarex.ru www.metarex.ru}
 */
class mxl_dict_dbgrid extends mxl_core_metadata{
	/**
	 * Функция создает объект грид, читая json файл.
	 * Файл должен иметь имя совпадающее с именем грида и расширение 
	 * указанное в {@link mxl_cfg_base::$json_file_ext}.
	 * @param string $metaDataName - имя грида
	 * @param mxl_core_params $params - динамические параметры
	 * @return mxl_dict_dbgrid
	 */
	public static function loadFromJSON($metaDataName, mxl_core_params $params){
		$fileName = mxl_core_metadata::getJSONFileName(mxl_cfg_path::$dict_grids,
				$metaDataName);
		try {
			$cfg = mxl_utils_json::readArrayFromFile($fileName, $params);

			$dbg = new mxl_dict_dbgrid($cfg['name'], $cfg['caption'], $cfg['tableName'],
					$cfg['primaryKey'],	$cfg['logicBaseName']);

			$dbg->width            = $cfg['options']['width'];
			$dbg->height           = $cfg['options']['height'];
		
			$dbg->selectByLeftJoin = $cfg['options']['selectByLeftJoin'];
			$dbg->readOnly         = $cfg['options']['readOnly'];
		
		
			if ((!$dbg->readOnly) && (isset($cfg['options']['validator']))) {
				$validatorName     = $cfg['options']['validator'];
				if ($validatorName){
					$MDSS = mxl_core_metadatas::instance();
				
					$dbg->validator = $MDSS->getMetaData(
							mxl_core_metadatas::$validator, $validatorName);
				}
			}
		
            if (isset($cfg['options']['renderer'])){
                $dbg->renderer = $cfg['options']['renderer'];
            }
		
			if (isset($cfg['options']['nextColOnEnter'])) {
				$dbg->nextColOnEnter   = $cfg['options']['nextColOnEnter'];
			}
			if (isset($cfg['options']['exportAble'])) {
				$dbg->setExportAble($cfg['options']['exportAble']);
			}
		
			$dbg->clicksToEdit     = $cfg['options']['clicksToEdit'];

			$manyRowsSaveMode      = $cfg['options']['manyRowsSaveMode'];
			if ($manyRowsSaveMode)
				$dbg->manyRowsSaveMode = $manyRowsSaveMode;
		
			$paginMode             = $cfg['options']['paginMode'];
			if ($paginMode)
				$dbg->addPagingBar($cfg['options']['pageStart'],$cfg['options']['pageSize']);
		
		
			$queryText = $cfg['queryText'];
			if ($queryText)
				$dbg->queryText = $queryText;
			
			$wh = $cfg['where'];
			if ($wh)
				$dbg->WH = $wh;
		
			$order = $cfg['order'];
			if ($order)
				$dbg->OC = $order;

			if (!isset($cfg['fields'])){
				throw new Exception();
				return false; 
			}

			$fields = $cfg['fields'];
			foreach ($fields as $field){
				$name = strtolower($field['n']);
	
				$fld = new mxl_dict_dbgridfield($name, $field['c'], $field['w'], 
						$field['dt'], $field['dl'], $field['a'], $field['mr']);
				if ( (($fld->getDataType() == 'edatetime')||($fld->getDataType() == 'date'))
						&&(isset($field['formatD'])) ){
					$fld->setFormatDate($field['formatD']);
				}
				if ( ($fld->getDataType() == 'edatetime')&&(isset($field['formatT'])) ){
					$fld->setFormatTime($field['formatT']);
				}
				
				if ( (($fld->getDataType() == 'float')||($fld->getDataType() == 'int'))&&
						(isset($field['formatN'])) ){
					$fld->setFormatNumber($field['formatN']);
				}
				$dbg->listFields[$name] = $fld;
			}
		
			if (isset($cfg['relations'])){
				$relations = $cfg['relations'];
				foreach ($relations as $relation){
					$lw = 0;
					if (isset($relation['lw']))
						$lw = $relation['lw'];
					$dbg->addRelation($relation['fn'], $relation['rl'], 
							$relation['cn'], $lw);
				}
			}

			if (isset($cfg['totals']['grid'])){
				$totalsGrid = $cfg['totals']['grid'];
				foreach ($totalsGrid as $total)
					$dbg->addTotalGrid($total['fn'],$total['tp'],$total['c']);
			}
			if (isset($cfg['totals']['footer'])){
				$totalsFooter = $cfg['totals']['footer'];
				foreach ($totalsFooter as $total)
					$dbg->addTotalFooter($total['fn'],$total['tp'],$total['c'],$total['h']);
			}
		
			if (isset($cfg['defaults'])){
				$defaults = $cfg['defaults'];
				foreach ($defaults as $def)
					$dbg->defaults[$def['fn']] = $def['df'];
			}

			$dbg->checkRelation();
        
            foreach ($dbg->listFields as $FI) {
                if ($FI->getAttribs()->isNU() && !$FI->getAttribs()->isRO() && ($FI->getCfgComboName() == ''))
            		$FI->getAttribs()->addAtribs ('|RO|');
            }
		} catch (Exception $e){
			$errMess = mxl_cfg_exceptions::$dict_dbgrid_loadFromJSON;
			if (strpos($errMess, '#VTINS1') !== false)
				$errMess = str_replace('#VTINS1', $fileName, $errMess);
			if (strpos($errMess, '#VTINS') !== false)
				$errMess = str_replace('#VTINS', $metaDataName, $errMess);
			
			throw new Exception($errMess .' '. $e->getMessage());
		}
		
		return $dbg;
	}
	
	/**
	 * Имя логической БД для работы грида.
	 * @var string
	 */
	private $logicBaseName;
	
	/**
	 * Имя грида
	 * @var string
	 */
	private $name;

	/**
	 * Заголовок грида, выводится в визуальной части.
	 * @var string
	 */
	private $caption;
	/**
	 * Имя базовой таблицы грида, необходимо для построения sql выражений.
	 * @var string
	 */
	private $tableName; 
	
	/**
	 * Имя поля первичного ключа
	 * @var string
	 */
	private $primKey;
	
	/**
	 * Текст макроса запроса
	 * @var string
	 */
	private $queryText;

	/**
	 * Статичный фильтр, задается в json файле грида
	 * @var string
	 */
	private $WH;
	/**
	 * Динамический фильтр, задается в виджет части. На сегодняшний момент
	 * пересылается в качестве параметра от Ajax к серверу php.
	 * @var string
	 */
	private $WC;
	
	/**
	 * Выражение для сортировки, задется в json-файле описания грида.
	 * @var string
	 */
	private $OC;
	
	/**
	 * Режим работы грида: "только чтение"
	 * @var boolean
	 */		
	private $readOnly;
	
	/**
	 * Режима обхода грида после редактирования по enter;
	 * true - переходит к редактированию следующего поля строки;
	 * false - перемещает фокус на следующую строку.
	 * 
	 * @var boolean
	 */
	private $nextColOnEnter;
	
	/**
	 * Валидатор, актуален для режима работы грида "только чтение" = false
	 * @var mxl_dict_validator
	 */
	private $validator;
	
	/**
	 * Имя файла рендерера, вызывается в процессе получения данных
	 * @var string
	 */
	private $renderer;

    /**
	 * Высота виджета грида
	 * @var int
	 */
	private $height;
	
	/**
	 * Ширина виджета грида
	 * @var int
	 */
	private $width;
	
	/**
	 * Режим работы грида: "Построение запросов с использованием left join".
	 * @var boolean
	 */
	private $selectByLeftJoin;

	/**
	 * Массив для хранения списка полей грида
	 * @var array()
	 */
	private $listFields;
	
	/**
	 * Режим работы грида: "Постраничный".
	 * @var boolean
	 */
	private $pagingMode;
	/**
	 * Начальная страница грида, активно при "Постраничном" режиме работы 
	 * {@link $pagingMode}
	 * @var int
	 */
	private $pageStart;
	/**
	 * Кол-во строк на странице грида, активно при "Постраничном" режиме работы 
	 * {@link $pagingMode} 
	 * @var int
	 */
	private $pageSize;

	/**
	 * Режим работы: "Итоги по гриду". В этом режиме отображаются итоги по гриду. 
	 * 
	 * Переводится в этот режим при использовании процедуры добавления итогов 
	 * по гриду.
	 * 
	 * @var boolean
	 */
	private $totalsGridMode;
	
	/**
	 * Массив итогов по гриду
	 * @var array()
	 */
	private $listTotalsGrid;
	
	/**
	 * Режим работы: "Отображать заголовок грида".
	 * @var boolean
	 */
	private $showGridHeader;
	
	/**
	 * Индикатор существования возможности экспорта данных грида
	 * @var boolean
	 */
	private $exportAble;
	
	/**
	 * Режим работы: "Итоги в футере грида". Итоги отображаются с помощью
	 * плагина ExtJS по видимой части грида (имееся в виду деление на страницы).
	 * 
	 * @var boolean
	 */
	private $totalsFooterMode;
	
	/**
	 * Массив итогов в фоотере грида. 
	 * @var array()
	 */
	private $listTotalsFooter;
	
	/**
	 * Задание кол-ва кликов кнопкой мыши для перехода в редактирование поля.
	 * Только для редактируемых гридов.
	 * 
	 * @var integer
	 */
	private $clicksToEdit;
	
	/**
	 * Режим работы: "сохранение множества строк"
	 * Если true - режим сохранения множества строк
	 * если false - режим сохранения по 1 строке
	 * @var boolean
	 */
	private $manyRowsSaveMode;
	
	/**
	 * Массив значений по умолчанию
	 * @var array
	 */
	private $defaults;
	
	/**
	 * Параметр говорящий был ли инициализирован массив значений по умолчанию.
	 * @var boolean
	 */
	private $defaultsPrepared;
	

	/**
	 * смещение от серверного времени в часах. 
	 * @var float
	 */
	private $timeShift;
	
	/**
	 * Конструктор класса. Создает объект класса и инициализирует начальными 
	 * значениями.
	 * @param $name - имя грида
	 * @param $caption - заголовок грида
	 * @param $tableName - имя таблицы
	 * @param $primKey - имя поля являющегося первичным ключом в таблице
	 * @param $logicBaseName - логическая БД
	 * @return mxl_dict_dbgrid
	 */
	function __construct($name, $caption, $tableName, $primKey, $logicBaseName){
		
		$this->name             = $name;
		$this->caption          = $caption;
		$this->tableName        = strtolower($tableName);
		$this->primKey          = strtolower($primKey);
		$this->logicBaseName    = $logicBaseName;
	    $this->height           = 200;
	    $this->width            = 400;
		
	    $this->selectByLeftJoin	= true;
		$this->readOnly         = false;
		$this->nextColOnEnter   = false;
		
		$this->listFields       = array();
		
		$this->pagingMode       = false;
		$this->totalsGridMode   = false;
		$this->totalsFooterMode = false;
		$this->showGridHeader   = false;
		$this->exportAble       = false;
		
		$this->listTotalsGrid   = array();
		$this->listTotalsFooter = array();
		
		$this->clicksToEdit     = 1;
		$this->manyRowsSaveMode = false;
		
		$this->defaultsPrepared = false; 
		$this->defaults         = array();
	}
		
	/**
	 * Деструктор грида.
	 * @return unknown_type
	 */
	function __destruct(){
		unset($this->listFields);
		unset($this->listTotalsGrid);
		unset($this->listTotalsFooter);
		unset($this->defaults);
	}

	/**
	 * Метод проверки логики отношений. Наличия обязательных полей.
	 * Наличие полей в связи. Наличие полей по которым строится связь.
	 * В случае успешной проверки возвращается true, в противном случае
	 * выбрасывается исключенеи соответствующее типу ошибки. 
	 * @return boolean
	 */
	private function checkRelation(){
		foreach ($this->listFields as $FI){
			/*
			 * Проверка наличия ключевого поля связи в таблице
			 */
			$relIdFieldName = $FI->getRelIdFieldFromRel();
			if ($relIdFieldName != ''){
				$exist = false;
				foreach ($this->listFields as $FIC)
					if ($relIdFieldName == $FIC->getName())
						$exist = true;
				if (! $exist){
					$errMess = mxl_cfg_exceptions::$dict_dbgrid_checkRelation_keyField;
					if (strpos($errMess, '#VTINS1') !== false)
						$errMess = str_replace('#VTINS1', $FI->getRel(), $errMess);
					if (strpos($errMess, '#VTINS') !== false)
						$errMess = str_replace('#VTINS', $relIdFieldName, $errMess);
			
					throw new Exception($errMess);
				}
			}
		}
	}
	
	/**
	 * Добавляет отношение с другими метаданными к полю.
	 * Например: отношение на выбор значений из другой таблицы по средствам
	 * комбобокса, или выбор значений из перечислений по средствам комбобокса.
	 * @param string $FieldName - имя поля к которому добавляется отношение
	 * @param string $relation - макрос отношения
	 * @param string $cfgComboName - имя комбобокса, который отвечает за задание значений
	 * @param int $listWidth - ширина списка комбобокса
	 */
	private function addRelation($FieldName, $relation = '', $cfgComboName = '',
			$listWidth = 0){
		$field = $this->getFieldByName($FieldName);
		$field->setRelation($relation, $cfgComboName);
		if ($listWidth > 0)
			$field->setListWidth($listWidth);
		$relIdFld = $field->getRelIdFieldFromRel();	
		if($relIdFld != ''){
			$field = $this->getFieldByName($relIdFld);
			$attr = $field->getAttribs();
			$attr->addAtribs(mxl_dict_fieldattribs::hidden.mxl_dict_fieldattribs::readOnly);
		}
	}
	
	/**
	 * Функция переводит грид в режим работы по страницам, передает номер 
	 * начальной страницы и размер страницы грида.
	 * 
	 * @param $pageStart - integer - начальная страница
	 * @param $pageSize - integer - размер страницы
	 */
	private function addPagingBar($pageStart = 0, $pageSize = 25){
		$this->pagingMode = true;
		$this->pageStart  = $pageStart;
		$this->pageSize   = $pageSize;
	}
	
	/**
	 * Функция для добавления итогов по гриду. Итоги добавляются в заголовок 
	 * грида и представляют собой итоги по гриду в целом, без учета деления на 
	 * страницы.
	 * При использовании грид переводится в режим работы с "Итоги по гриду" и
	 * "Отображать заголовок грида".
	 *    
	 * @param $fieldName - имя поля, по которому считается итог
	 * @param $type - тип итога, может быть: 'sum', 'count*', 'count', 'min', 
	 * 		'max'.
	 * @param $caption - заголовок итога
	 */
	private function addTotalGrid($fieldName, $type, $caption){
		$this->totalsGridMode = true;
		$this->showGridHeader = true;
		
		//проверка на наличие поля
		$fi = $this->getFieldByName($fieldName);
		$name = $fi->getName();
		
		$total = new mxl_dict_dbgridtotalgrid($name, $type, $caption);
		$this->listTotalsGrid[$name] = $total;
	}
	
	/**
	 * Функция для добавления итогов в футер грида. Итоги представляют собой 
	 * итоги по гриду с учетом деления на страницы.
	 * При использовании грид переводится в режим работы "Итоги в футере грида"
	 * и "Отображать заголовок грида".
	 * 
	 * @param $fieldName - имя поля, по которому считается итог
	 * 
	 * @param $type - тип итога, может быть: 'sum', 'count', 'min', 
	 * 		'max'.
	 * 
	 * @param $caption - Макрос заголовка итога.
	 * Значения макроса разделяются '|';
	 * Макрос для вставки значения #VTINS;
	 * Возможны три ситуации указания значений:
	 * 1. идентификатор значения не заполнен - общий заголовок
	 * 2. идентифиакфтор #NVAL= - значение определяющие текст отображения для 
	 * ситуации когда итог не имеет значение.
	 * 3. "значение=" - для указанного значения будет выведен этот текст
	 * 	 
	 * @param $hint - Текст всплывающего сообщения.
	 * 
	 */
	private function addTotalFooter($fieldName, $type, $caption = '', $hint = ''){
		$this->totalsFooterMode = true;
		$this->showGridHeader   = true;
		
		//проверка на наличие поля
		$fi = $this->getFieldByName($fieldName);
		$name = $fi->getName();
		
		$total = new mxl_dict_dbgridtotalfooter($name, $type, $caption, $hint);
		$this->listTotalsFooter[$name] = $total;
	}
		
	/**
	 * Функция возвращает строку итогов по гриду, строится на основе добавленных
	 * процедурой addTotalGrid итогов по гриду.
	 * Предназначена для использования внутри библиотеки.
	 * Если грид не находится в режиме работы "Итоги по гриду" то выбрасывается 
	 * исключение.
	 * @throws Exception {@link mxl_cfg_exceptions::$dict_dbgrid_getTotalGridStr}
	 * @return string
	 */
	public function getTotalGridStr(){
		if (!$this->isTotalsGridMode()){
			$errMess = mxl_cfg_exceptions::$dict_dbgrid_getTotalGridStr;
			if (strpos($errMess, '#VTINS') !== false)
				$errMess = str_replace('#VTINS', $this->name, $errMess);
			throw new Exception($errMess);
		}
		
		$MXL_LDBS = new mxl_core_logicbases();
		$MXL_DB = $MXL_LDBS->getLogicBaseInst($this->getLogicBaseName());
							
		$totalGridStr = '';
		$sqlTotalFields = '';
		foreach($this->listTotalsGrid as $TG){
			$FI = $this->getFieldByName($TG->getName());

			if ($TG->getType() == 'count*')
				$sqlTotalFields .= 'count(*) as countt_'.$TG->getName();
			else{
				$nameField = ($FI->getRelField() ? 
						($FI->getRelSyn() ? $FI->getRelSyn().'.' : '')
								.$FI->getRelField() : 
						'a.'.$FI->getName());
				
				$sqlTotalFields .= $TG->getType().'('.$nameField.') as '
						.$TG->getType().'_'.$TG->getName();
			}
			$sqlTotalFields .= ', ';
		}
		#echo $sqlTotalFields;
		
		$query = $this->getSelectSQL();
		$query = str_replace('select ', 'select '.$sqlTotalFields, $query);
		
		$result = $MXL_DB->sql_query($query);
		$totals_row = $MXL_DB->sql_fetchrow($result);
		$MXL_DB->sql_freeresult($result);
	
		foreach($this->listTotalsGrid as $TG){
			$fldName = ($TG->getType() == 'count*' ? 'countt': $TG->getType())
					.'_'.$TG->getName();
			
			$totalGridStr .= $TG->getCaption().' \''.$totals_row[$fldName].'\'; ';
		}
		$totalGridStr = substr($totalGridStr, 0, strlen($totalGridStr)- 2).'.';
		return $totalGridStr;
	}

	/**
	 * функция возвращает список полей таблицы в формате для построения 
	 * sql выражений.
	 * @param $return_not_update - включать поля с атрибутом 'NU'
	 * @param $return_with_syn - возвращать с синонимом.
	 * @return string
	 */
	function getListFieldsStr($return_not_update = false, $return_with_syn = true){
		$r = '';
		foreach ($this->listFields as $FI){
			if ($return_not_update || 
					(!$return_not_update && !$FI->getAttribs()->isNU() )){
				$syn = $return_with_syn ? 
						(($FI->getRelSyn() ? 
								$FI->getRelSyn() : 
								'a').'.') : 
						'';
				$r .= $syn.$FI->getName().',';
			}
		}
		if (strlen($r) > 1){
			$r = substr($r, 0, strlen($r) - 1);
		}
		return $r;
	}
	
	/**
	 * Проверяет есть ли в гриде поля из других таблиц. Если есть то возвращает
	 * true.
	 * @return boolean
	 */
	function isRelationTable(){
		if ($this->queryText) 
			return true;

		$findNU = false;
		
		foreach ($this->listFields as $FI)
			if ($FI->getAttribs()->IsNU()){
				$findNU = true;
				break;
			}
		return $findNU;
	}	
	
	/**
	 * Возвращает массив полей имеющих синоним равный $syn.
	 * @param $syn - синоним
	 * @return array()
	 */
	function getRelListFieldsByRelSyn($syn){
		$fields = array();
		foreach ($this->listFields as $FI)			
			if ($FI->getAttribs()->isNU() && $FI->getRelSyn() == $syn)
				$fields[] = $FI;
		return sizeof($fields) ? $fields : false;
	}
	
	/**
	 * возвращает масив полей которые имеют связь с другой таблицей
	 * ключ массива - ключевое поле по которому строится связь,
	 * значение - поле в котором определена связь
	 * @return array
	 */
	function getRelKeysFieldsList(){
		$relKeys = array();
		foreach ($this->listFields as $FI){
			$relFieldId = $FI->getRelIdFieldFromRel(); 
			if ($relFieldId)
				$relKeys[$relFieldId] = $FI;
		}
		return $relKeys;
	}
	
	/**
	 * подготавливает массив значений по умолчанию, если он не был подготовлен.
	 * Процесс подготовки заключается в том, что функция разбирает массив 
	 * значений по умолчанию и если встречает поля со связами к другим
	 * таблицам, то заполяет все поля из этой же связи
	 */
	public function preprareDefaults(){
		if ($this->defaultsPrepared)
			return;
		
		$relKeys = $this->getRelKeysFieldsList();
		$addDefaults = array();
		foreach ($this->defaults as $defKey => $defVal){
			if (isset($relKeys[$defKey])){
				$keyField = $relKeys[$defKey];
				$row = $this->getLinkFieldsValues($defKey, $keyField, $defVal);

				$flist = $this->getRelListFieldsByRelSyn($keyField->getRelSyn());
				foreach ($flist as $f)
					$addDefaults[$f->getName()] = $row[$f->getRelField()];
			}
		}
		$this->defaults = array_merge($addDefaults, $this->defaults);
	}
	
	/**
	 * Возвращает поля имеющие атрибут $attribute, если &$flist == -1 то
	 * дублиреут поля в качестве массива в $flist.
	 * @param $attribute - атрибут, по которому вчведется поиск
	 * @param $return_with_syn - возвращать поля с синонимом
	 * @param $flist - дублирующий массив полей
	 * @param $array_key_is_name - формировать $flist как ассоциированный
	 * 		массив, где ключем выступает имя поля 
	 * @return string
	 */
	function getListFieldsByAttr($attribute = '', $return_with_syn = true, 
			&$flist = -1, $array_key_is_name = false){
		$r = '';

		$fill_in_arr = false;
		
		if ($flist !== -1)
			$fill_in_arr = true;
			
		foreach ($this->listFields as $FI){
			$find = false;
			if ($attribute != '') {
				if     (mxl_dict_fieldattribs::isROS($attribute))
					$find = $FI->getAttribs()->isRO();
				elseif (mxl_dict_fieldattribs::isHDS($attribute))
					$find = $FI->getAttribs()->isHD();
				elseif (mxl_dict_fieldattribs::isNUS($attribute))
					$find = $FI->getAttribs()->isNU();
			} else
				$find = true;
			
			if ($find){	
				$syn = $return_with_syn ? 
						(($FI->getRelSyn() ? $FI->getRelSyn() : 'a').'.') : 
						'';
				$rel_name = mxl_dict_fieldattribs::isNUS($attribute) ? 
						($FI->getRelField() ? 
								($FI->getRelField() .' as ' .$FI->getName()) : 
								 $FI->getName() ) : 
						'';
				
				$r .= $syn. ($rel_name ? $rel_name : $FI->getName()).',';
				
				if ($fill_in_arr)
					if (!$array_key_is_name)
						$flist[] = $FI;
					else
						$flist[$FI->getName()] = $FI;
			}
		}
		if (strlen($r) > 1)	{
			$r = substr($r, 0, strlen($r) - 1);
		}
		
		return $r;
	}
	
	/**
	 * Возвращает масисв полей имеющих тип $dataType, если $dataType = '*'
	 * то возвращаются все поля. Если не установлен параметр $return_not_update
	 * то исключаются поля с атрибутом 'NU'.
	 * @param $dataType - тип данных
	 * @param $return_not_update - возвращать поля с атрибутом 'NU'
	 * @return unknown_type
	 */
	function getListFieldsByDataType($dataType, $return_not_update = false) {
		$res = array();
		foreach ($this->listFields as $FI) {
			if (($FI->getDataType() == $dataType || $dataType == '*') 
					&& (	$return_not_update 
						|| (!$return_not_update && !$FI->getAttribs()->isNU())))
				$res[] = $FI;
		}
		return sizeof($res) ? $res : false;
	}

	/**
	 * Получить поле по имени. Если поле не найдето то выбрасывается исключение.
	 * @param $name - имя поля
	 * @throws Exception {@link mxl_cfg_exceptions::$dict_dbgrid_getFieldByName}
	 * @return mxl_dict_dbgridfield
	 */
	public function getFieldByName($name){
		$name = strtolower($name);
		if (!array_key_exists($name, $this->listFields)){
			$errMess = mxl_cfg_exceptions::$dict_dbgrid_getFieldByName;
			if (strpos($errMess, '#VTINS') !== false)
				$errMess = str_replace('#VTINS', $name, $errMess);
			throw new Exception($errMess);
			return false;
		}
	
		return $this->listFields[$name];
	}
	
	/**
	 * Возвращает Итог в футере по имени поля, если таковой имеется.
	 * @param $fieldName - имя поля для которого ищется итог
	 * @return mxl_dict_dbgridtotalfooter
	 */
	public function getTotalFooterByFieldName($fieldName){
		$fieldName = strtolower($fieldName);

		if (!array_key_exists($fieldName, $this->listTotalsFooter))
			return;
		else
			return $this->listTotalsFooter[$fieldName];
	}
	
	/**
	 * Типизирует значение для поля
	 * @param $nameField - имя поля к которому принадлежит значение
	 * @param $value - значение, которое необходимо типизировать
	 * @param $to_sql - режим типизации "для sql выражений"
	 * @return unknown_type
	 */
	public function typenaze($nameField, $value, $to_sql = true)	{
		$FI = $this->getFieldByName($nameField);
		if (is_null($value))
            return $value;
		if ($to_sql){ 		
			switch($FI->getDataType()) {
				case 'date':
					return $value;
					break;
				case 'string':
					return '\'' . $value . '\'';
					break;
				case 'boolean':
					return intval($value);
					break;
				default:
					return $value;
					break;
			}
		} else {
			switch($FI->getDataType()) {
				case 'int':
					return (int) $value;
					break;
				case 'float':
					return (double) $value;
					break;
				case 'date':
                    return (int) mxl_utils_edatetime::datetime_to_int($value);
					break;
				case 'edatetime':
					return (int) $value;
					break;
				case 'string':
					return (string) $value;
					break;
			}
		}
	}
	
	/**
	 * Происзодит типизацию асоциированного массива, ключ которого имя поля.
	 * @param $arr - массив для типизации
	 * @return boolean
	 */
	function typenazeArray(&$arr){
		foreach ($arr as $key => $value){
			$arr[$key] = $this->typenaze($key, $value, false);
		}
		return true;
	}
	
	/**
	 * Функция которая проверяет поле грида и приводит его в 
	 * соответствие с полем в таблице БД
	 * @param string $fieldName - имя поля грида 
	 * @return string|boolean
	 */
	private function checkFieldAndAssign($fieldName){
		$res = false;
		try{
			$FI = $this->getFieldByName($fieldName);
			if ($FI !== false)
				$res = $fieldName;
		} catch (Exception $e){
			$flist = $this->getListFieldsByDataType('edatetime');
			if ($flist)
				foreach ($flist as $field) {
					$name = $field->getName();
					if (($name.'_date' == $fieldName) ||($name.'_time' == $fieldName)){
						$res = $name;
					}
				}
		}
		 
		return $res;
		
	}
	
	/**
	 * функция опредления SELECT выражения на основе данных DBGrid
	 * 
	 * @param $sort - string - поле для сортировки, подготовленое widget, имеет 
	 * 		самый высокий приоритет при определении сортировки в sql запросе
	 * @param $dir - string - Тип сортировки: ASC, DESC
	 * @return sql str
	 */
	public function getSelectSQL($sort = '', $dir = ''){
		$FI    = $this->getListFieldsStr();
		$table = $this->getTableName();
		
		$query = '';
		$WC    = $this->getWC();
		
		if ($sort != '') {
			$sort = $this->checkFieldAndAssign($sort);
			$OC = ($sort !== false) ? ($sort . ' ' . $dir) : $this->getOC();
		}
		else
			$OC = $this->getOC();
		
		if ($this->isRelationTable()) {
			$query = $this->queryText;

			if ($this->selectByLeftJoin) {
				if ($query == '') {
					$NU_FI	= $this->getListFieldsByAttr('|NU|', true, $flist);

					$query = ' select {fkUPDATE}'.($NU_FI ? ','.$NU_FI : '')."\n".
							 ' from {TABLE} a'. "\n";

					$sql_addenda1 = $sql_addenda2 = '';

					$size = sizeof($flist);
					if ($size) {
						foreach ($flist as $f)
							if ($f->getRelTable() != '' && $f->getRelSyn() != ''
									 && $f->getRel())
								$sql_addenda1 .= '	left join '.$f->getRelTable()
										.' '. $f->getRelSyn().' on '.$f->getRel()."\n";
					}

					$WC	.= $this->getWH() ? (($WC ? ' and ' : '').$this->getWH()) : '';
				
					$query .= $sql_addenda1 . ($WC ? ' where {WH}'."\n" : '') 
							. '{ORDER_BY}';
				}else
					$WC .= ($this->getWH()?(($WC ? ' and ': '').$this->getWH()):'');
			
				$replaceOC = (strpos($query,'{ORDER_BY}') !== false) ?
				 		'order by '.($OC ? $OC : 'a.'.$this->getPrimKey()) :
						false;
				
				//Cинтаксический анализ sql с помощью str_replace
				//Определяем search
				$search = array('{fkUPDATE}', '{TABLE}', '{WH}');

				if ($replaceOC != false)
					$search[] = '{ORDER_BY}';
				//***Определяем search

				//Определяем replace
				$replace = array($FI, $table, $WC);

				if ($replaceOC != false)
					$replace[] = $replaceOC;
				//***Определяем replace
		
				$query = str_replace($search, $replace, $query);
				//***Парсинг sql с помощью str_replace
			} else {
				if ($this->queryText == ''){
					
					$NU_FI	= $this->getListFieldsByAttr('|NU|', true, $flist);
	
					$query = ' select {fkUPDATE}' . ($NU_FI ? ','.$NU_FI : '') 
							.' from {TABLE} a';
	
					$size = sizeof($flist);
				
					$sql_addenda1 = $sql_addenda2 = '';
					if ($size) {
						foreach ($flist as $f) {
						if ($f->getRelTable() != '' && $f->getRelSyn() != '' 
								&& $f->getRel()) {
							$sql_addenda1 .= ','.$f->getRelTable().' '.$f->getRelSyn();
							$sql_addenda2 .= ($sql_addenda2 != '') ? 
									' and '.$f->getRel() : 
									$f->getRel();
							}
						}
					}			
				
					$query .= $sql_addenda1.' where '.$sql_addenda2.' {WH} {ORDER_BY}';
				}
				if ($WC || $this->getWH()) {
					$WC	 = ($WC            ? ' and '.$WC            : '');
					$WC	.= ($this->getWH() ? ' and '.$this->getWH() : '');
				}
	
				$replaceOC = (strpos($query,'{ORDER_BY}') !== false) ?
				 		'order by '. ($OC ? $OC : 'a.'.$this->getPrimKey()) :
						false;
			
				//Cинтаксический анализ sql с помощью str_replace
				//Определяем search
				$search = array('{fkUPDATE}', '{TABLE}', '{WH}');
			
				if ($replaceOC != false)
					$search[] = '{ORDER_BY}';
				//***Определяем search
			
				//Определяем replace
				$replace = array($FI, $table, $WC);
			
				if ($replaceOC != false)
					$replace[] = $replaceOC;
				//***Определяем replace
		
				$query = str_replace($search, $replace, $query);
				//***Парсинг sql с помощью str_replace
			}
		} else {
			$query = 'select '. $FI .' from '. $table . ' a ';
	
			$WC .= ($this->getWH() ? (($WC ? ' and ': '').$this->getWH()) : '');
			
			$query .= $WC ? ' where '   .$WC : '';
			$query .= $OC ? ' order by '.$OC : '';
		}
		//echo $query."\n";
		return $query;
	}
	
	/**
	 * Функция для возвращения части sql выражения отвечающее за ограничение
	 * по кол-ву строк на отбор значений.
	 * @param $start - с какой строки начинать
	 * @param $limit - сколько строк отобрать
	 * @return string
	 */
	private function getLimitSQL($start, $limit){
		$query = ' LIMIT '. (int) $start.', '. (int) $limit;
		return $query;
	}
	
	/**
	 * Функция для построения запроса на получение кол-ва строк для запроса
	 * 
	 * @param $selectSql - sql запрос для которого необходимо получить кол-во строк 
	 * @return string
	 */
	private function getTotalRowsSQL($selectSql){
		$sqlTotals = 'select count(*) as cnt from('.$selectSql.') as aliasM99';
		return $sqlTotals;
	}
	
	/**
	 * Функция для получения кол-ва строк для запроса $selectSql
	 * 
	 * @param $selectSql - запрос для которого необходимо получить кол-во строк
	 * @return integer - кол-во строк
	 */
	private function getTotalRows($selectSql){
		$MXL_LDBS = new mxl_core_logicbases();
		$MXL_DB = $MXL_LDBS->getLogicBaseInst($this->getLogicBaseName());
				
		$totals = -1;
		
		$sqlTotals = $this->getTotalRowsSQL($selectSql);
		
		$result = $MXL_DB->sql_query($sqlTotals);
		$totalsr = $MXL_DB->sql_fetchrow($result);
		$MXL_DB->sql_freeresult($result);
			
		$totals = $totalsr['cnt'];
				
		return $totals;
	}

	/**
	 * Функция получения данных грида в формате json и их передачи на 
	 * клиентскую часть.
	 * Функция формирует sql запрос на основании данных указанных в свойствах 
	 * класса, обращается к бд с целью получения данных и формирует текст ответа 
	 * в формате json.  
	 * Вывод осуществляется при помощи функции echo.
	 * 
	 * @TODO необходим обработчик ошибок получаемых от sql запросов 
	 * 
	 * @param int $start
	 * @param int $limit
	 * @param string $sort
	 * @param string $dir
	 * @return string
	 */
	public function getJSON($start = 0, $limit = 0, $sort = '', $dir = ''){
		$MXL_LDBS = new mxl_core_logicbases();
		$MXL_DB = $MXL_LDBS->getLogicBaseInst($this->getLogicBaseName());
		
		$totals = -1;
		
		$query = $this->getSelectSQL($sort, $dir);
		if ($limit > 0){
			$totals = $this->getTotalRows($query);
			$query .= $this->getLimitSQL($start, $limit);
		}
	
		header('text/javascript');
		#echo $query;
		$result = $MXL_DB->sql_query($query);
		
		
		//->----------------------------------------EXCLUDE E-DATETIME FROM ROW
		$flist_edatetime = array();
		$flist = $this->getListFieldsByDataType('edatetime');
		if ($flist)
			foreach ($flist as $field) {
				$name = $field->getName();
				$flist_edatetime[$name] = 1;
			}
		//-<----------------------------------------EXCLUDE E-DATETIME FROM ROW
		$flist_date = array();
		$flist = $this->getListFieldsByDataType('date');
		if ($flist)
			foreach ($flist as $field) {
				$name = $field->getName();
				$flist_date[$name] = 1;
			}

        $flist_datetime = array();
		$flist = $this->getListFieldsByDataType('datetime');
		if ($flist)
			foreach ($flist as $field) {
				$name = $field->getName();
				$flist_datetime[$name] = 1;
			}

		$cnt = 0;
		while ($r = $MXL_DB->sql_fetchrow($result)) {
			if ($cnt == 0){ //проверка полей на присутствие в результируеющем наборе
                $errors = '';
                foreach ($this->getListFields() as $FI){
                    if (!array_key_exists($FI->getName(), $r)){
                        $errors .= ($errors?', ':'').$FI->getName();
                    }
                }
                if ($errors){
                    $errMess = mxl_cfg_exceptions::$dict_dbgrid_getJson_haventFields;
                    if (strpos($errMess, '#VTINS') !== false)
                    $errMess = str_replace('#VTINS', $errors, $errMess);
            
                    throw new Exception($errMess);
                }
            }
            $cnt++;
		
			//-------------------------------------------------apply time shift
			foreach ($flist_edatetime as $name => $value)
				if (isset($r[$name])){
					$r[$name] = mxl_utils_edatetime::applyTimeShift($r[$name], 
							$this->timeShift);
				}
				
			//->------------------------------------EXCLUDE E-DATETIME FROM ROW
			foreach ($flist_edatetime as $name => $value)
				if (isset($r[$name]))
					mxl_utils_edatetime::excludeFromRow($r, $name);

            //->------------------------------------TRANSFORM INT TO DATE
			foreach ($flist_date as $name => $value)
				if (isset($r[$name])){
                    if ($r[$name]!= '0')
                        $r[$name] = (string) strftime('%m/%d/%Y', $r[$name]);
                    else
                        $r[$name] = '';
                }
			//->------------------------------------TRANSFORM INT TO DATETIME
			foreach ($flist_datetime as $name => $value)
				if (isset($r[$name])){
                    if ($r[$name]!= '0')
                        $r[$name] = (string) strftime('%m/%d/%Y', $r[$name])
                                    . ' ' .  strftime('%H:%M',    $r[$name]);
                    else
                        $r[$name] = '';
                }
                
            //вызов рендерера для строки
            if (isset($this->renderer)){
                $r = mxl_dict_renderer::execute_renderer ($this->renderer, 
                        mxl_core_metadatas::instance()->getCurrParams(), $r);
            }

			$rows[] = $r;
		}
		$MXL_DB->sql_freeresult($result);

		if ($cnt > 0){
			echo '({"total":"'.($totals == -1 ? $cnt : $totals).'","rows":'
					.json_encode($rows).'})';
		}else
			echo '({"total":"0", "rows":[]})';
	}
	
	/**
	 * Метод для обновления значения поля в БД по значению ключевого поля.
	 * @param $key - значение для ключевого поля
	 * @param $field - поле, которое требуется обновить
	 * @param $value - значение для обновляемого поля
	 * @param $successParamExt - расширение json ответа в случае успешного 
	 * 		обновления поля 
	 */
	public function updateField($key, $field, $value, $successParamExt = ''){
		$MXL_LDBS = new mxl_core_logicbases();
		$MXL_DB = $MXL_LDBS->getLogicBaseInst($this->getLogicBaseName());
				
		header ('text/javascript');
		if (($key == '')||($field == '')||($value == '')) {
			echo '{failure:true}';
			return false;
		}
	
		$query = ' update '. $this->getTableName()
				.' set '.$field.'='.$this->typenaze($field, $value)
				.' where '.$this->getPrimKey().'='
						  .$this->typenaze($this->getPrimKey(), $key);

		$MXL_DB->return_on_error = true;
		$res = $MXL_DB->sql_query($query);
		$MXL_DB->return_on_error = false;
		
		$totalGridStr = '';
		if ($this->isTotalsGridMode())
			$totalGridStr = ' total_str:"'.$this->getTotalGridStr().'"';

		if ($res)
			echo '{failure:false'
					.($totalGridStr    != '' ? ','.$totalGridStr    : '')
					.($successParamExt != '' ? ','.$successParamExt : '').'}';
		else
			echo '{failure:true}';
	} 
	
	/**
	 * Функция ищет поле c $idFieldName в отношении и возвращает поле, в котором 
	 * найден этот $idFieldName.
	 * @param $idFieldName - имя ключевого поля для поиска 
	 * @return mxl_dict_dbgridfield
	 */
	function getFieldByLinkIdFieldName($idFieldName){
		foreach ($this->listFields as $FI){
			$relIdFieldName = $FI->getRelIdFieldFromRel();
			if (strtolower($relIdFieldName) == strtolower($idFieldName)){
				return $FI;
			}
		}
	}
	
	/**
	 * Возвращает значения связанных полей отношения к внешним таблицам
	 * @param $key - имя ключевого поля в таблице к которой построено отношение
	 * 		(связь)
	 * @param $relField - поле c с описанием отношения (связи) к другой таблице
	 * 		в гриде
	 * @param $value - значение для ключевого поля
	 * @return array()
	 */
	private function getLinkFieldsValues($key, $relField, $value){
		$table	  = $relField->getRelTable();
		$flist	  = $this->getRelListFieldsByRelSyn($relField->getRelSyn());
		$nu_fields = '';
		
		$MXL_LDBS = new mxl_core_logicbases();
		$MXL_DB = $MXL_LDBS->getLogicBaseInst($this->getLogicBaseName());
			
		foreach ($flist as $field){
			$nu_fields .= ($nu_fields != '' ? ',' : '');
			$nu_fields .= $field->getRelField();
		}
		$query = 'select '.$nu_fields. ' from ' . $table
				. ' where '.$key.'='.$value;

		$MXL_DB->return_on_error = true;
		$res = $MXL_DB->sql_query($query);
		$MXL_DB->return_on_error = false;
	
		$row = $MXL_DB->sql_fetchrow($res);

		if ($row)
			return $row;
		else
			return ;
	}
	
	/**
	 * Функция для получения значений свзязанных полей и формирования ответа в
	 * формате json.
	 * Вывод осуществляется функцией echo.
	 * @param $fieldName - имя поля, которое содержит отношение 
	 * @param $key - имя ключевого поля в связанной таблице
	 * @param $value - значение ключевого поля
	 */
	function updateLinkFields($fieldName, $key, $value) {
		$key = strtolower($key);

		header('text/javascript');
		
		if (($key == '') || ($fieldName == '')){
			echo '{failure:true}';
			return false;
		}
		
		$keyField  = $this->getFieldByName($fieldName);
		if ($value != ''){
			$row = $this->getLinkFieldsValues($key, $keyField, $value);
			if ($row){
				$cnt = count($row);
				echo '({failure:false,"total":"'.$cnt.'","fields":'.json_encode($row).'})';
			} else
				echo '{failure:true}';
		} else {
			$flist = $this->getRelListFieldsByRelSyn($keyField->getRelSyn());
			$cnt   = 0;
			$resp  = '';
			foreach ($flist as $field){
				$resp .= ($resp != '' ? ',' : '');
				$resp .= '"'.$field->getRelField().'":""';
				$cnt ++;
			}
			$resp = '{'.$resp.'}';
			echo '({failure:false,"total":"'.$cnt.'","fields":'.$resp.'})';
		}
	}
	
	/**
	 * Функция для сохранения значений строки грида в БД. Строка передается
	 * в виде асоциированного массива.
	 * Если значение ключевого поля пустое, то происходит вставка строки.
	 * В случае успешного сохранения, функция возвращает значение первичного 
	 * ключа.
	 * @param mxl_core_logicbase $MXL_LDB - объект подключения к БД.
	 * @param $key - значение ключевого поля
	 * @param $arr_data - ассоциированный масив со значениями строки
	 * @return int|false
	 */
	private function saveRowInDB(mxl_core_logicbase $MXL_LDB, $key, $arr_data){
		$MXL_DB = $MXL_LDB->getMXL_DB();
		
		/////ADD E-DATETIME TO ROW
		$flist_edatetime = array();
		$flist = $this->getListFieldsByDataType('edatetime');
		if ($flist)
			foreach ($flist as $field) {
				$name = $field->getName();
				$flist_edatetime[$name] = 1;
			}

		foreach ($flist_edatetime as $name => $value) 
			if (isset($arr_data[$name.'_date']) && isset($arr_data[$name.'_time']))
				mxl_utils_edatetime::includeToRow($arr_data, $name);
		////*****ADD E-DATETIME TO ROW	
	
		foreach ($flist_edatetime as $name => $value) 
			if (isset($arr_data[$name]))
				$arr_data[$name] = mxl_utils_edatetime::cancelTimeShift(
						$arr_data[$name], $this->timeShift);
				
		////Exclude Fields which not found in grid fields
		$flist = array();
		foreach ($this->getListFields() as $FI){
			if (!$FI->getAttribs()->isNU()){
				$flist[$FI->getName()] = $FI;
			}
		}
			
		$row = array();
		foreach ($arr_data as $name => $val){
			if (isset($flist[$name]))
				$row[$name] = $val;
		}
		$arr_data = $row;
		////*****Exclude Fields which not found in grid fields
	
		////Exclude Not Update fields & Primary Key
		$NU_FI = array();
		$this->getListFieldsByAttr('|NU|', true, $NU_FI, true);
		
		$row = array();
		foreach ($arr_data as $name => $val) {
			if (!isset($NU_FI[$name]) && $name !== $this->getPrimKey())
				$row[$name] = $val;
		}
		$arr_data = $row;
		////*****Exclude Not Update fields & Primary Key

		
		$this->typenazeArray($arr_data);
		//****проверка типа полей таблицы базы данных, при попытке записи в поле int пустого значения, 
		// присваиваем значению 0
		// данное действие облегчит жизнь, когда поле грида является ENUM, значение отображается строковое, а
		// в таблицу базы должно писаться целое число, ссылка на другую таблицу, если очистить поле грида, в 
		// запрос передается пустая строка, что приводит к ошибке записи
		$query = 'SELECT COLUMN_NAME as cName, DATA_TYPE as dType'
				.' FROM information_schema.COLUMNS'
				.' WHERE TABLE_NAME="'.$this->getTableName() . '"';
		$res = $MXL_DB->sql_query($query);
		while ($nRes = $MXL_DB->sql_fetchrow($res)){
			if ($nRes['dType'] == 'int' && isset($arr_data[$nRes['cName']]) && $arr_data[$nRes['cName']] == ''){
				$arr_data[$nRes['cName']] = 0;
			}
		}
		
		if ($key == '')
			$query = 'insert into '.$this->getTableName().' '
					.$MXL_DB->sql_build_array('INSERT', $arr_data);
		else
			$query = ' update ' . $this->getTableName()
					.' set ' . $MXL_DB->sql_build_array('UPDATE', $arr_data)
					.' where '.$this->getPrimKey().'='
							  .$this->typenaze($this->getPrimKey(), $key);
		
		$res = $MXL_DB->sql_query($query);
		if ($res){
			if ($key == '')
				$key = $MXL_DB->sql_nextid();
			return $key;
		} else
			return false;
	}
	
	/**
	 * Функция сохранения одной строки. Сохраняет строку в БД и формирует 
	 * ответ в формате json.
	 * @param $key - значение ключевого поля
	 * @param $data - данные строки в формате json
	 * @param $successParamExt - параметры расширения ответа в случае успеха
	 */
	public function saveRow($key, $data, $successParamExt = ''){
		$MXL_LDBS = new mxl_core_logicbases();
		$MXL_LDB = $MXL_LDBS->getLogicBase($this->getLogicBaseName());
		
		header ('text/javascript');
		
		$arr_data = json_decode(stripslashes($data), true); // associative arrays
		if (isset($this->validator))
			$v_res = mxl_dict_validator_checker::execute($arr_data, 
				$this->validator, mxl_core_metadatas::instance()->getCurrParams(),
				$this);
		else
			$v_res = true;
			
		if ($v_res === true) {
			$record_id = $this->saveRowInDB($MXL_LDB, $key, $arr_data);
			if ($record_id !== false){
				$totalGridStr = '';
				if ($this->isTotalsGridMode())
					$totalGridStr = ' total_str: "'.$this->getTotalGridStr().'"';
	
		
				echo '{failure:false'.($key == '' ? ', record_id:'.$record_id: '')
						.($totalGridStr    != '' ? ','.$totalGridStr    : '')
						.($successParamExt != '' ? ','.$successParamExt : '').'}';
			} else
				echo '{failure:true}';
		} else if ($v_res !== true){
			echo '{failure:true, validate:'.json_encode($v_res).'}';
		} else
			echo '{failure:true}';
		
	}
	
	/**
	 * Функция сохранения массива строк. Сохраняет строки в БД и формирует 
	 * ответ в формате json.
	 * @param $keys - массив значений ключевых полей
	 * @param $datas - массив данных строк в формате json
	 * @param $successParamExt - параметры расширения ответа в случае успеха
	 */
	public function saveRows($keys, $datas, $successParamExt = ''){
		$MXL_LDBS = new mxl_core_logicbases();
		$MXL_LDB = $MXL_LDBS->getLogicBase($this->getLogicBaseName());
		
		header ('text/javascript');

		$failure = true;
		
		$arr_keys  = json_decode(stripslashes($keys), false); 
		//$arr_datas = json_decode(stripslashes($datas), true);
		$arr_datas = json_decode($datas, true);

		if (isset($this->validator))
			$v_res = mxl_dict_validator_checker::execute_rows($arr_datas, 
				$this->validator, mxl_core_metadatas::instance()->getCurrParams(),
				$this);
		else
			$v_res = true;
			
		if ($v_res === true) {
			$rowIsSaved = array();
			$insert_ids = array();
			$count = count($arr_keys);
			for($i = 0; $i < $count; $i++){
				$key = $arr_keys[$i];
				$data = $arr_datas[$i];
		
				$record_id = $this->saveRowInDB($MXL_LDB, $key, $data);
			
				if ($record_id !== false){
					$rowIsSaved[$i] = true;
					$insert_ids[$i] = ($key == '' ? $record_id : '');
					$failure = false;
				} else {
					$rowIsSaved[$i] = false;
					$insert_ids[$i] = '';
				}
			}
		}
		
		if ($failure == false){
			$totalGridStr = '';
			if ($this->isTotalsGridMode())
				$totalGridStr = ' total_str: "'.$this->getTotalGridStr().'"';
	
			echo '{failure:false'
					.', row_is_saved: '.json_encode($rowIsSaved).''
					.', insert_ids: '  .json_encode($insert_ids).''
					.($totalGridStr    != '' ? ','.$totalGridStr    : '')
					.($successParamExt != '' ? ','.$successParamExt : '').'}';
		} else if ($v_res !== true){
			echo '{failure:true, validate:'.json_encode($v_res).'}';
		} else
			echo '{failure:true}';
		
	}
	
	/**
	 * Функция удаления строки из БД. Удаляет строку из БД и формирует ответ
	 * в формате json.
	 * @param $key - значение ключевого поля.
	 * @param $successParamExt - расширение параметров ответа в случае успешного
	 * 		удаления
	 */
	function deleteRow($key, $successParamExt = ''){
		$MXL_LDBS = new mxl_core_logicbases();
		$MXL_DB = $MXL_LDBS->getLogicBaseInst($this->getLogicBaseName());
				
		header ('text/javascript');
		if ($key == ''){
			echo '{failure:true}';
			return false;
		}
	
		$v_res = true;
		$res   = false;
		
		if ( (isset($this->validator)) && ($this->validator->getBeforeDelete()) ){
			$v_res = mxl_dict_validator_snippet::execute_validate(
					$this->validator->getBeforeDelete(), 
					mxl_core_metadatas::instance()->getCurrParams(), array(), $key);
		}
		
		if ($v_res === true){
			$query = 'delete from '. $this->getTableName();
	
			if ($this->getWC())
				$query .= ' where '.$this->getWC();
		
			if ($key) {
				if ($this->getWC())
					$query .= ' and ';
				else
					$query .= ' where ';
				$query .= $this->getPrimKey().'='
						.$this->typenaze($this->getPrimKey(), $key);
			}
	
			$MXL_DB->return_on_error = true;
			$res = $MXL_DB->sql_query($query);
			$MXL_DB->return_on_error = false;
		
			$totalGridStr = '';
			if ($this->isTotalsGridMode())
				$totalGridStr = ' total_str:"'.$this->getTotalGridStr().'"';
		}
	
		if ($res){
			echo '{failure:false'
					.($successParamExt != '' ? ','.$successParamExt : '')
					.($totalGridStr    != '' ? ','.$totalGridStr    : '').'}';
		} else if ($v_res !== true){
			echo '{failure:true, validate:'.json_encode($v_res).'}';
		} else {
			echo '{failure:true}';
		}
	}
	
	/**
	 * Функция для возвращения имени грида.
	 * @return string
	 */
	public function getName(){
		return $this->name;
	}
	
	/**
	 * Функция для возвращения имени ключевого поля в гриде
	 * @return string
	 */
	public function getPrimKey(){
		return $this->primKey;
	}
	
	/**
	 * Функция для возвращения заголовка грида.
	 * @return string
	 */
	public function getCaption(){
		return $this->caption;
	}
	
	/**
	 * Функция для возвращения имени таблицы в БД
	 * @return string
	 */
	public function getTableName(){
		return $this->tableName;
	}
	
	/**
	 * Функция для возвращения высоты грида
	 * @return int
	 */
	public function getHeight(){
		return $this->height;
	}
	
	/**
	 * Функция для возвращения ширины грида
	 * @return int
	 */
	public function getWidth(){
		return $this->width;
	}
	
	/**
	 * Функция для возвращения списка полей грида.
	 * @return array()
	 */
	public function getListFields(){
		return $this->listFields;
	}
	
	/**
	 * Функция для возвращения списка итогов в футере.
	 * @return array()
	 */
	public function getListTotalsFooter(){
		return $this->listTotalsFooter;
	}
	
	/**
	 * Функция возвращает ответ на вопрос находится ли грид в режиме 
	 * только для чтения.
	 * @return boolean
	 */
	public function isReadOnly(){
		return $this->readOnly;
	}
	
	/**
	 * Функция возвращает true, если включен режим перехода к редактированию 
	 * следующего поля строки по enter.
	 * @return boolean
	 */
	public function isNextColOnEnter() {
		return $this->nextColOnEnter;
	}
	
	/**
	 * Функция для возвращения значения статического фильтра.
	 * @return string
	 */
	public function getWH(){
		return $this->WH;
	}

	/**
	 * Функция для возвращения значения динамического фильтра.
	 * @return string
	 */
	public function getWC(){
		return $this->WC;
	}
	
	/**
	 * Функция для установки значения динамического фильтра.
	 */
	public function setWC($WC){
		$this->WC = $WC;
	}
	
	/**
	 * Функция для возвращения значения сортировки.
	 * @return string
	 */
	public function getOC(){
		return $this->OC;
	}
	
	/**
	 * Функция возвращает ответ на вопрос находится ли грид в режиме 
	 * постраничной работы.
	 * @return boolean
	 */
	public function isPagingMode(){
		return $this->pagingMode;
	}
	
	/**
	 * Функция для возвращения размера страницы. Актуальна при постраничном
	 * режиме работы грида.
	 * @return int
	 */
	public function getPageSize(){
		return $this->pageSize;
	}
	
	/**
	 * Функция для возвращения начальной страницы. Актуальна при постраничном
	 * режиме работы грида.
	 * @return int
	 */
	public function getPageStart(){
		return $this->pageStart;
	}
	
	/**
	 * Функция возвращает ответ на вопрос находится ли грид в режиме 
	 * работы итогов по гриду.
	 * @return boolean
	 */
	public function isTotalsGridMode(){
		return $this->totalsGridMode;
	}

	/**
	 * Функция возвращает ответ на вопрос находится ли грид в режиме 
	 * показа панели заголовка грида. В этот режим грид переводится
	 * если есть итоги в футере грида или итоги по гриду в целом.
	 * @return boolean
	 */
	public function isShowGridHeader(){
		return $this->showGridHeader;
	}
	
	/**
	 * Функция возвращает ответ на вопрос есть ил возможность эакспорта 
	 * даных грида. Возможность экспорта задётся в словаре грида.
	 * @return boolean
	 */
	public function isExportAble(){
		return $this->exportAble;
	}
	
	/**
	 * Функция устанавливает значение индикатора возможности экспорта
	 * данных грида. Если экспорт возможен, то устанавливается в true 
	 * индикатор отображения панели заголовка грида.
	 * @param $exportAble - значение индикатора
	 */
	public function setExportAble($exportAble) {
		$this->exportAble = $exportAble;
		if ($exportAble) $this->showGridHeader = true;
	}
	
	/**
	 * Функция возвращает ответ на вопрос находится ли грид в режиме 
	 * работы итогов в футере грида.
	 * @return boolean
	 */
	public function isTotalsFooterMode(){
		return $this->totalsFooterMode;
	}
	
	/**
	 * Функция для возвращения кол-ва кликов мышкой для перехода в режим 
	 * редактирования.
	 * @return int
	 */
	public function getClicksToEdit(){
		return $this->clicksToEdit;
	}
	
	/**
	 * Функция для возвращения имени логической БД, для таблицы которой построен
	 * грид.
	 * @return string
	 */
	public function getLogicBaseName(){
		return $this->logicBaseName;
	}
	
	/**
	 * Функция возвращает ответ на вопрос находится ли грид в режиме 
	 * сохранения множества строк.
	 * @return boolean
	 */
	public function isManyRowsSaveMode(){
		return $this->manyRowsSaveMode;
	}
	
	/**
	 * Функция для возвращения списка значений по умолчанию.
	 * @return string
	 */
	public function getDefaults(){
		if ($this->defaults)
			return $this->defaults;
		else
			return '';
	}
	
	/**
	 * Функция для Добавления суффикса к таблице грида.
	 * @return string
	 */
	public function addTableNameSuf($tableNameSuf){
		$this->tableName = $this->tableName.$tableNameSuf;
	}
	
	/**
	 * Устанавливает смещение времени от серверного в часах. Применяется для
	 * всех полей типа edatetime.
	 * @param float $timeShift
	 */
	public function setTimeShift($timeShift){
		$this->timeShift = $timeShift;
	}
}
?>