<?php
/**
 * Файл для описания абстрактного класса списка, {@link mxl_dict_list}.
 * @package mxl_dict
 * @author merlin <merlin@ersys.ru>
 * @copyright Copyright (c) 2010, ООО НПО Метарекс
 */
/**
 * Абстрактный класс для хранения общей информации по "списку".
 * Содержит базовые свойства и методы для "списков".
 * 
 * @package mxl_dict
 * @author merlin <merlin@ersys.ru>
 * @copyright Copyright (c) 2010, ООО НПО Метарекс, {@link http://www.metarex.ru www.metarex.ru}
 */
abstract class mxl_dict_list extends mxl_core_metadata{
	/**
	 * Вид списка - один список с выбором множества значений
	 * @var string
	 */
	public static $lkMultiselect  = 'multiselect';
	/**
	 * Вид списка - два списка: доступный и список отобранных значений
	 * @var string
	 */
	public static $lkItemselector = 'itemselector';
	
	/**
	 * Функция загрузки списка из json-файла. В зависемости от типа списка
	 * загружается либо mxl_dict_listenum::loadListFromCfg либо 
	 * mxl_dict_listdb::loadListFromCfg.
	 * @param string $metaDataName - имя метаданного
	 * @param mxl_core_params $params - параметры загрузки
	 */
	public static function loadFromJSON($metaDataName, mxl_core_params $params){
		$fileName = mxl_core_metadata::getJSONFileName(mxl_cfg_path::$dict_lists,
				$metaDataName);
		
		$cfg = mxl_utils_json::readArrayFromFile($fileName, $params);

		$type = $cfg['type'];
		$list = '';
		
		if ($type == 'enum')
			$list = mxl_dict_listenum::loadListFromCfg($cfg, $list);
		else if ($type == 'db')
			$list = mxl_dict_listdb::loadListFromCfg($cfg, $list);
		else if ($type == 'dbsql')
			$list = mxl_dict_listdbsql::loadListFromCfg($cfg, $list);
			
		return $list;
	}
	
	/**
	 * Загрузка базовых зарактеристик списка - общих для всех видов списков.
	 * @param array $cfg - массив зарактеристик
	 * @param mxl_dict_list $list - объект список
	 * @throws Exception
	 */
	public static function loadListFromCfg($cfg, $list = ''){
		if (isset($cfg['kind'])){
			if (    ($cfg['kind'] != mxl_dict_list::$lkMultiselect)&& 
					($cfg['kind'] != mxl_dict_list::$lkItemselector)){
				throw new Exception("Указан не правильный вид: ".$cfg['kind']);
				return false;		
			}
				
			$list->setKind($cfg['kind']);
		}
				
		if (isset($cfg['title']))
			$list->setTitle($cfg['title']);
		if (isset($cfg['fieldLabel']))
			$list->setFieldLabel($cfg['fieldLabel']);
		if (isset($cfg['clearButton']))
			$list->setClearButton($cfg['clearButton']);
		
		if (isset($cfg['commonWidth']))
			$list->setCommonWidth($cfg['commonWidth']);
		if (isset($cfg['elementWidth']))
			$list->setElementWidth($cfg['elementWidth']);
		if (isset($cfg['height']))
			$list->setHeight($cfg['height']);
			
        if (isset($cfg['renderer'])){
            $list->renderer = $cfg['renderer'];
        }
		
		return $list;		
	}
	
	/** 
 	 * может принимать значения: 
 	 * 		multiselect  - Список со множественным выбором
 	 * 		itemselector - Два списка с перемещением выбранных значений во второй список
	 * @var string
 	 */
	private $kind;
	/**
	 * Имя поля, со значением комбобокса
	 * @var string
	 */
	private $valueField;
	/**
	 * Имя поля, которое будет отображаться
	 * @var string
	 */
	private $displayField;
	
	/**
	 * Заголовок размещающийся на списком в заголовке панели.
	 * Если не указан, то заголовок панели не отображается. 
	 * @var string
	 */
	private $title;
	
	/**
	 * Надпись, которая размещается перед списком.
	 * Если не указана, то не отображается. 
	 * @var string
	 */
	private $fieldLabel;
	
	/**
	 * Создавать ли кнопку очистки выбора в списке.
	 * @var boolean
	 */
	private $clearButton;
	
	/**
	 * Общая ширина виджета 
	 * @var int
	 */
	private $commonWidth;
	/**
	 * Ширина элемента. Если список Itemselector то ширина каждого списка.
	 * @var int
	 */
	private $elementWidth;
	/**
	 * Высота клиентской части виджета (панели на которой размещаются списки)
	 * @var int
	 */
	private $height; 
	
	/**
	 * Имя файла рендерера, вызывается в процессе получения данных
	 * @var string
	 */
	private $renderer;

	/**
	 * Конструктор класса {@link mxl_dict_list}.
	 * @param $name - имя списка
	 * @param $valueField - имя поля с ключем
	 * @param $displayField - имя поля со значением
	 * @return mxl_dict_list
	 */
	public function __construct($name, $valueField, $displayField){
		parent::__construct($name);	
		$this->kind          = mxl_dict_list::$lkMultiselect;
		
		$this->valueField    = strtolower($valueField);
		$this->displayField  = strtolower($displayField);
		
		$this->commonWidth   = 400;
		$this->elementWidth  = 150;
		$this->height        = 150;
	}
	
	/**
	 * Метод для получения имени ключевого поля списка.
	 * @return string
	 */
	public function getValueField(){
		return $this->valueField;
	}
	
	/**
	 * Метод для получения имени поля со значением списка.
	 * @return string
	 */
	public function getDisplayField(){
		return $this->displayField;
	}
	
	/**
	 * Абстрактная функция получения данных для заполнения списка в 
	 * формате json и их передачи на клиентскую часть.
	 * Вывод осуществляется при помощи функции echo.
	 */
	abstract function getJSON();
	
	/**
	 * Метод для устаноки вида "Списка" 
	 *   multiselect  - Список со множественным выбором
 	 * 	 itemselector - Два списка с перемещением выбранных значений во второй список
	 * @param string $kind
	 */
	public function setKind($kind){
		$this->kind = $kind;
	}
	
	/**
	 * метод для получения вида "Списка".
	 * @return string
	 */
	public function getKind(){
		return $this->kind;
	}
	
	/**
	 * Метод для установки заголовока размещающегося на списком в заголовке панели.
	 * Если не указан, то заголовок панели не отображается. 
	 * @param string $title
	 */
	public function setTitle($title){
		$this->title = $title;
	}
	
	/**
	 * метод для получения заголовка.
	 * @return string
	 */
	public function getTitle(){
		return $this->title;
	}
	
	/**
	 * метод для установки значения Надписи размещающейся перед списком. 
	 * Если не указана, то не отображается. 
	 * @param string $fieldLabel
	 */
	public function setFieldLabel($fieldLabel){
		$this->fieldLabel = $fieldLabel;
	}
	
	/**
	 * метод для получения Надписи размещающейся перед списком.
	 * @return string
	 */
	public function getFieldLabel(){
		return $this->fieldLabel;
	}
	
	/**
	 * Метод для установки свойства создать кнопку очистки выбранных значений. 
	 * @param boolean $clearButton
	 */
	public function setClearButton($clearButton){
		$this->clearButton = $clearButton;
	}
	
	/**
	 * метод для получения значения свойства создавать ли кнопку очистки 
	 * выбранных значений.
	 * @return boolean
	 */
	public function getClearButton(){
		return $this->clearButton;
	}
	/**
	 * Метод для установки ошбщей ширины виджета
	 * @param int $commonWidth
	 */
	public function setCommonWidth($commonWidth){
		$this->commonWidth = $commonWidth;
	}
	
	/**
	 * Метод для получения общей ширины виджета
	 * @return int
	 */
	public function getCommonWidth(){
		return $this->commonWidth;
	}

	/**
	 * Метод установки ширины элемента
	 * @param int $elementWidth
	 */
	public function setElementWidth($elementWidth){
		$this->elementWidth = $elementWidth;
	}
	
	/**
	 * Метод получения значения ширины элемента.
	 * @return int
	 */
	public function getElementWidth(){
		return $this->elementWidth;
	}

	/**
	 * Метод для установки клиентсвой ширины виджета.
	 * @param int $height
	 */
	public function setHeight($height){
		$this->height = $height;
	}
	
	/**
	 * Метод для получения клиентской ширины виджета
	 * @return int
	 */
	public function getHeight(){
		return $this->height;
	}

    /**
	 * Возвращает имя файла рендерера
	 * @return string
	 */
	public function getRenderer(){
		return $this->renderer;
	}
	
}

?>