<?php
/**
 * Файл описывающий класс "Выполнения валидации", {@link mxl_dict_validator_checker}.
 * @package mxl_dict
 * @author merlin <merlin@ersys.ru>
 * @copyright Copyright (c) 2009, ООО НПО Метарекс
 */
/**
 * Класс "Выполнения валидации".
 * Содержит методы запускающие процесс проверки данных при помощи валидатора
 * 
 * @package mxl_dict
 * @author merlin <merlin@ersys.ru>
 * @copyright Copyright (c) 2009, ООО НПО Метарекс, {@link http://www.metarex.ru www.metarex.ru}
 */
class mxl_dict_validator_checker{
	/**
	 * Массив правил поддерживаемых валидатором
	 * @var array
	 */
	public static $possibleRules = array(
			'common' => array(
					'required',   
					'periodMember',   
					'checkEMail',   
					'checkLogin',   
					'checkPassword',   
					'match',   
					'checkType'), 
			'dbgrid' => array(
					'required',   
					'periodMember',
					'checkType', 
					'unique',
					'uniqueLR',					
					'checkEMail',   
					'checkLogin',   
					'checkPassword',   
					'match'));
	
	/**
	 * Статический метод для проверки одной строки на соответствие правилам
	 * валидации. В случае успешной проверки возвращает true. Если проверка
	 * не пройдена то возвращается сообщение о нарушение правила.
	 * @param array $row - строка передаваемая для проверки
	 * @param mxl_dict_validator $validator - правила валидатора, которые 
	 * 		необходимо проверить
	 * @param mxl_core_params $params - динамические параметры 
	 * @param mxl_core_metadata $metadata - указатель на объект метаданного
	 * 		которому принадлежит строка
	 * @return boolean|string
	 */
	public static function execute ($row, mxl_dict_validator $validator,
			mxl_core_params $params, mxl_core_metadata $metadata){
		
		$result = true;
		
		if ($validator->getBeforeValidate()){
			$result = mxl_dict_validator_snippet::execute_validate(
					$validator->getBeforeValidate(), $params, $row);
			if ($result !== true)
				return $result;
		}

		foreach ($validator->getRules() as $section => $rules){
			if ($section == 'common'){
				$result = mxl_dict_validator_rulescommon::execute($rules, $row);
				if ($result !== true)
					return $result;
			}else if ($section == 'dbgrid'){
				$result = mxl_dict_validator_rulesdbgrid::execute($rules, $row,
						$metadata);
				if ($result !== true)
					return $result;
			}
		}
		
		if ($validator->getAfterValidate()){
			$result = mxl_dict_validator_snippet::execute_validate(
					$validator->getAfterValidate(), $params, $row);
			if ($result !== true)
				return $result;
		}
		
		return $result;
	}
	
	/**
	 * Статический метод для проверки массива строк на соответствие правилам
	 * валидации. В случае успешной проверки возвращает true. Если проверка
	 * не пройдена то возвращается сообщение о нарушение правила.
	 * @param array $row - строка передаваемая для проверки
	 * @param mxl_dict_validator $validator - правила валидатора, которые 
	 * 		необходимо проверить
	 * @param mxl_core_params $params - динамические параметры 
	 * @param mxl_core_metadata $metadata - указатель на объект метаданного
	 * 		которому принадлежит строка
	 * @return boolean|string
	 */
	public static function execute_rows ($rows, mxl_dict_validator $validator,
			mxl_core_params $params, mxl_core_metadata $metadata){
		$result = '';
		
		if ($validator->getBeforeAllValidate()){
			$result = mxl_dict_validator_snippet::execute_validate(
					$validator->getBeforeAllValidate(), $params, '', -1, $rows);
			if ($result !== true)
				return $result;
		}
		
		foreach ($rows as $key => $row){
			$result_l = mxl_dict_validator_checker::execute($row, $validator,
					$params, $metadata);
			if ($result_l !== true)
				$result .= $result_l."\n";		
		}
		
		if ($result == '')
			$result = true;
			
		return $result;
	}
	
}
?>
