<?php
/**
 * Файл для класса правил валидации секции common, {@link mxl_dict_validator_rulescommon}.
 * 
 * @package mxl_dict_validator
 * @author merlin <merlin@ersys.ru>
 * @copyright Copyright (c) 2009, ООО НПО Метарекс
 */
/**
 * Класс правил валидации секции common.
 * Правила секции common предназначены для проверки значений всех типов метаданных.
 * Запустить проверку правильности данных можно с помощью статической функции 
 * {@link mxl_dict_validator_rulescommon::execute()}.
 * 
 * @package mxl_dict_validator
 * @author merlin <merlin@ersys.ru>
 * @copyright Copyright (c) 2009, ООО НПО Метарекс, {@link http://www.metarex.ru www.metarex.ru}
 */
class mxl_dict_validator_rulescommon {
	
	/**
	 * Статическая функция вызывает методы для проверки массива данных $row в соответствии 
	 * с правилами $rules. Возвращает true в случае успешной проверки, иначе сообщение 
	 * о нарушении правила. 
	 * 
	 * @param array $rules - массив правил: ключевое значение определяет метод обработки,
	 * 		значение по ключу содержит правило проверки 
	 * @param array $row - массив данных для проверки
	 * @return boolean | string
	 */
	public static function execute ($rules, $row) {
		foreach ($rules as $ruleName => $rule) {
			if ($ruleName == 'required') {
				$result = mxl_dict_validator_rulescommon::required($rule, $row);
				if ($result !== true)
					return $result;
			} else if ($ruleName == 'periodMember'){
				$result = mxl_dict_validator_rulescommon::periodMember($rule, $row);
				if ($result !== true)
					return $result;
			} else if ($ruleName == 'checkEMail'){
				$result = mxl_dict_validator_rulescommon::checkEMail($rule, $row);
				if ($result !== true)
					return $result;
			} else if ($ruleName == 'checkLogin'){
				$result = mxl_dict_validator_rulescommon::checkLogin($rule, $row);
				if ($result !== true)
					return $result;
			} else if ($ruleName == 'checkPassword'){
				$result = mxl_dict_validator_rulescommon::checkPassword($rule, $row);
				if ($result !== true)
					return $result;
			} else if ($ruleName == 'match'){
				$result = mxl_dict_validator_rulescommon::match($rule, $row);
				if ($result !== true)
					return $result;
			} else if ($ruleName == 'checkType'){
				$result = mxl_dict_validator_rulescommon::checkType($rule, $row);
				if ($result !== true)
					return $result;
			}
		} 
		return true;
	}
	
	/**
	 * Метод проверки полей, необходимых к заполнению. 
	 * Возвращает true в случае успешной проверки, иначе сообщение о нарушении правила.
	 * 
	 * @param string $rule - параметр для передачи правила проверки, проверяемые
	 * 		поля передаются через ';', если необходимо указать заголовок 
	 * 		поля, который будет отображаться при выводе сообщений по этому полю,
	 * 		то в параметр задается через ':'. До ':' идет имя поля, после заголовок.
	 * 		пример: 'paramToCheck1:CaptionParam1;paramToCheck2;...' 
	 * @param array $row - массив для проверки на наличие значений
	 * @return boolean | string
	 */
	public static function required($rule, $row){
		$fieldsToCheck = explode(';', $rule);
		$result = '';
		foreach ($fieldsToCheck as $field){
			$caption = '';
			if (strpos($field, ':') !== false){
				$x = explode(':', $field);
				$field = trim($x[0]);
				$caption = trim($x[1]);
				unset($x);
			}
			else
				$caption = trim($field);
			if ( ($field != '') && ((!isset($row[$field])) || ($row[$field] == '')) )
				$result .= ($result != ''?',':'').'"'.$caption.'"';
		}
		if ($result != '')
			$result = '<br>'.mxl_cfg_messages::$dict_validator_rulescommon_required
					.':<br>'.$result.'.';
		else
			$result = true;
		return $result;
	}
	
	/**
	 * Вспомогательный метод для разбора строки правила вида:
	 * 'min1<field1:caption1<=max1', 'field2<max2', 'field3:caption1>=min3' или т.п. 
	 * Возвращает массив параметров правила: 
	 * 		max - максимальная граница интервала, 
	 * 		min - минимальная граница интервала, 
	 * 		maxstrict - true, если неравенство при максимальном значении строгое,
	 * 		minstrict - true, если неравенство при минимальном значении строгое,
	 * 		caption - заголовок поля,
	 * 		field - название поля.
	 * Если один из параметров (max, min) не найден, возвращается ''. 
	 * Если не найден параметр caption, ему присваивается значение параметра field.
	 * 
	 * @param string $rule - правило для одного поля
	 * @return array
	 */
	private static function parseIntervalRule($rule) {
		$max       = '';
		$min       = '';
		$maxstrict = true;
		$minstrict = true;
		$caption   = '';
		$field     = $rule;
		if (strpos($field, '<') !== false){
			$x = explode('<', $field);
			if (count($x) == 2) {
				$field = $x[0];
				$pos = strpos($x[1], '='); 
				if ($pos === false) {
					$max = $x[1] == '' ? null : $x[1];
				} else if ($pos == 0) {
					$buf = substr($x[1], 1);
					$max = $buf == '' ? null : $buf;
					$maxstrict = false;
				}
				unset($x);
			} else if (count($x) == 3) {
				$field = $x[1];
				$pos = strpos($field, '=');
				if ($pos === false) {
					$min = $x[0] == '' ? null : $x[0];	
				} else if ($pos == 0) {
					$field = substr($field, 1);
					$min = $x[0] == '' ? null : $x[0];
					$minstrict = false;
				}
				$pos = strpos($x[2], '='); 
				if ($pos === false) {
					$max = $x[2] == '' ? null : $x[2];
				} else if ($pos == 0) {
					$buf = substr($x[2], 1);
					$max = $buf == '' ? null : $buf;
					$maxstrict = false;
				}
				unset($x);
			}
		}
		if (strpos($field, '>') !== false){
			$x = explode('>', $field);
			if (count($x) == 2) {
				$field = $x[0];
				$pos = strpos($x[1], '='); 
				if ($pos === false) {
					$min = $x[1] == '' ? null : $x[1];
				} else if ($pos == 0) {
					$buf = substr($x[1], 1);
					$min = $buf == '' ? null : $buf;
					$minstrict = false;
				}
				unset($x);
			}
		}
		if (strpos($field, ':') !== false){
			$x = explode(':', $field);
			$field = $x[0];
			$caption = $x[1];
			unset($x);
		} else $caption = $field;
		$result = array('max' => trim($max), 
				'min'         => trim($min), 
				'maxstrict'   => trim($maxstrict), 
				'minstrict'   => trim($minstrict), 
				'caption'     => trim($caption), 
				'field'       => trim($field) );
		return $result;		
	}
	
	/**
	 * Метод проверки принадлежности значений полей промежутку допустимых значений
	 * Возвращает true в случае успешной проверки, иначе сообщение о нарушении правила.
	 * 
	 * @param string $rule - параметр для передачи правила проверки, проверяемые
	 * 		поля с границами интервала передаются через ';'.
	 * 		Если указывается одна из границ интервала, то сперва записывается имя поля,
	 * 		затем знак соответствующего неравенства, а затем значение границы.
	 * 		Если указываются обе границы интервала допустимых значений, то они записываются
	 * 		последовательно: минимальная граница, знак неравенства, имя поля, знак неравенства,
	 * 		максимальная граница.
	 * 		Проверяемые неравенства могут быть строгими (>, <) или не строгими (>=, <=).
	 * 		Если необходимо указать заголовок поля, который будет отображаться при выводе 
	 * 		сообщений по этому полю, то в параметр задается через ':'. До ':' идет имя поля, 
	 * 		после заголовок.
	 * 		пример: 'minvalue1<field1:caption1<=maxvalue1;field2<maxvalue2;field3:caption3>=minvalue3;...'
	 * @param array $row - массив для проверки принадлежности значений интервалу 
	 * 		допустимых значений
	 * @return boolean | string
	 */
	public static function periodMember($rule, $row){
		$fieldsToCheck = explode(';', $rule);
		$result = '';
		foreach ($fieldsToCheck as $field){
			$arr       = mxl_dict_validator_rulescommon::parseIntervalRule($field);
			
			$max       = $arr['max'];
			$min       = $arr['min'];
			$maxstrict = $arr['maxstrict'];
			$minstrict = $arr['minstrict'];
			$caption   = $arr['caption'];
			$field     = $arr['field'];
			if ($field != '' && isset($row[$field])) {
				if (trim($min) != '') {
					if ($minstrict && $row[$field] <= $min) {
						$result .= ($result != ''?',':'').'"'.$caption.'" > '.$min;
					} else if (!$minstrict && $row[$field] < $min) {
						$result .= ($result != ''?',':'').'"'.$caption.'" >= '.$min;
					}
				}
				if (trim($max) != '') {
					if ($maxstrict && $row[$field] >= $max) {
						$result .= ($result != ''?',':'').'"'.$caption.'" < '.$max;
					} else if (!$maxstrict && $row[$field] > $max) {
						$result .= ($result != ''?',':'').'"'.$caption.'" <= '.$max;	
					}
				} 
			}
		}
		if ($result != '')
			$result = '<br>'.mxl_cfg_messages::$dict_validator_rulescommon_periodMember
					.':<br>'.$result.'.';
		else
			$result = true;
		return $result;
	}
	
	/**
	 * Метод валидации e-mail адреса.
	 * Возвращает true в случае успешной проверки, иначе сообщение о нарушении правила.
	 * 
	 * @param string $rule - параметр для передачи правила проверки, проверяемые
	 * 		поля передаются через ';', если необходимо указать заголовок 
	 * 		поля, который будет отображаться при выводе сообщений по этому полю,
	 * 		то в параметр задается через ':'. До ':' идет имя поля, после заголовок.
	 * 		пример: 'paramToCheck1:CaptionParam1;paramToCheck2;...' 
	 * @param array $row - массив данных для валидации e-mail адреса 
	 * @return boolean | string
	 */
	public static function checkEMail($rule, $row){
		$fieldsToCheck = explode(';', $rule);
		$result = '';
		foreach ($fieldsToCheck as $field) {
			$caption = '';
			if (strpos($field, ':') !== false) {
				$x = explode(':', $field);
				$field   = trim($x[0]);
				$caption = trim($x[1]);
				unset($x);
			} 
			else
				$caption = trim($field);
			if ($field != '' && isset($row[$field]) &&
					!preg_match('/^[a-z][-\._a-z0-9]*@[a-z][-\._a-z0-9]*\.[a-z0-9]{2,5}$/i', $row[$field])) {
				$result .= ($result != ''?',':'').'"'.$caption.'"';
			}
		}
		if ($result != '')
			$result = '<br>'.mxl_cfg_messages::$dict_validator_rulescommon_checkEMail
					.':<br>'.$result.'.';
		else
			$result = true;
		return $result;
	}
	
	/**
	 * Метод валидации логина.
	 * Возвращает true в случае успешной проверки, иначе сообщение о нарушении правила.
	 * Логин может содержать латинские символы, цифры, '_', '.', '-'. Первый символ 
	 * логина должен быть буквой или цифрой.
	 * По умолчанию проверяется, чтобы в логине было не менее 1 символа, но не более 20,
	 * но границы допустимых размеров строки логина могут быть заданы правилом.
	 * 
	 * @param string $rule - параметр для передачи правила проверки, проверяемые
	 * 		поля с границами интервала допустимой длины строки передаются через ';'.
	 * 		Если указывается одна из границ интервала, то сперва записывается имя поля,
	 * 		затем знак соответствующего неравенства, а затем значение границы.
	 * 		Если указываются обе границы интервала допустимых значений, то они записываются
	 * 		последовательно: минимальная граница, знак неравенства, имя поля, знак неравенства,
	 * 		максимальная граница.
	 * 		Проверяемые неравенства могут быть строгими (>, <) или не строгими (>=, <=).
	 * 		Если необходимо указать заголовок поля, который будет отображаться при выводе 
	 * 		сообщений по этому полю, то в параметр задается через ':'. До ':' идет имя поля, 
	 * 		после заголовок.
	 * 		пример: 'field1;field2:caption2>=minlength2;minlength3<field3<=maxlength3;...'
	 * @param array $row - массив для валидации логина
	 * @return boolean | string
	 */
	public static function checkLogin($rule, $row){
		$fieldsToCheck = explode(';', $rule);
		$result = '';
		foreach ($fieldsToCheck as $field) {
			$arr = mxl_dict_validator_rulescommon::parseIntervalRule($field);
			if (is_null($arr['max'])) {
				$maxlength = 20;
				$maxstrict = false;
			} else {
				$maxlength = $arr['max'];
				$maxstrict = $arr['maxstrict'];
			}
			if (is_null($arr['min'])) {
				$minlength = 1;
				$minstrict = false;	
			} else {
				$minlength = $arr['min'];
				$minstrict = $arr['minstrict'];
			}
			$caption = $arr['caption'];
			$field   = $arr['field'];
			if ($field != '' && isset($row[$field])) {
				if (!is_null($minlength)) {
					if ($minstrict && strlen($row[$field]) <= $minlength) {
						$result .= ($result != ''?',':'').'"'.$caption.'" '
							.mxl_cfg_exceptions::$dict_validator_rulescommon_checkLogin1.'> '
							.$minlength;
					} else if (!$minstrict && strlen($row[$field]) < $minlength) {
						$result .= ($result != ''?',':'').'"'.$caption.'" '
							.mxl_cfg_exceptions::$dict_validator_rulescommon_checkLogin1.'>= '
							.$minlength;
					}
				}
				if (!is_null($maxlength)) {
					if ($maxstrict && strlen($row[$field]) >= $maxlength) {
						$result .= ($result != ''?',':'').'"'.$caption.'" '
							.mxl_cfg_exceptions::$dict_validator_rulescommon_checkLogin1.'< '
							.$maxlength;
					} else if (!$maxstrict && strlen($row[$field]) > $maxlength) {
						$result .= ($result != ''?',':'').'"'.$caption.'" '
							.mxl_cfg_exceptions::$dict_validator_rulescommon_checkLogin1.'<= '
							.$maxlength;
					}
				}
				if (!preg_match('/^[-\._a-z0-9]*$/i', $row[$field])) {
					$result .= ($result != ''?',':'').'"'.$caption.'" '
						.mxl_cfg_exceptions::$dict_validator_rulescommon_checkLogin2;
				} else if (!preg_match('/^[a-z0-9][-\._a-z0-9]*$/i', $row[$field])) {
					$result .= ($result != ''?',':'').'"'.$caption.'" '
						.mxl_cfg_exceptions::$dict_validator_rulescommon_checkLogin3;
				}
			}
		}
		if ($result != '')
			$result = '<br>'.mxl_cfg_messages::$dict_validator_rulescommon_checkLogin
					.':<br>'.$result.'.';
		else
			$result = true;
		return $result;
	}
	
	/**
	 * Метод валидации пароля.
	 * Возвращает true в случае успешной проверки, иначе сообщение о нарушении правила.
	 * Пароль может содержать любые символы, кроме кириллицы и спецсимволов (пробел, табуляция)
	 * По умолчанию пароль может быть пустым, но не может содержать более 20 символов,
	 * но границы допустимых размеров строки пароля могут быть заданы правилом.
	 * 
	 * @param string $rule - параметр для передачи правила проверки, проверяемые
	 * 		поля с границами интервала допустимой длины строки передаются через ';'.
	 * 		Если указывается одна из границ интервала, то сперва записывается имя поля,
	 * 		затем знак соответствующего неравенства, а затем значение границы.
	 * 		Если указываются обе границы интервала допустимых значений, то они записываются
	 * 		последовательно: минимальная граница, знак неравенства, имя поля, знак неравенства,
	 * 		максимальная граница.
	 * 		Проверяемые неравенства могут быть строгими (>, <) или не строгими (>=, <=).
	 * 		Если необходимо указать заголовок поля, который будет отображаться при выводе 
	 * 		сообщений по этому полю, то в параметр задается через ':'. До ':' идет имя поля, 
	 * 		после заголовок.
	 * 		пример: 'field1;field2:caption2>=minlength2;minlength3<field3<=maxlength3;...'
	 * @param array $row - массив для валидации пароля
	 * @return boolean | string
	 */
	public static function checkPassword($rule, $row){
		$fieldsToCheck = explode(';', $rule);
		$result = '';
		foreach ($fieldsToCheck as $field) {
			$arr       = mxl_dict_validator_rulescommon::parseIntervalRule($field);
			if (is_null($arr['max'])) {
				$maxlength = 20;
				$maxstrict = false;
			} else {
				$maxlength = $arr['max'];
				$maxstrict = $arr['maxstrict'];
			}
			if (is_null($arr['min'])) {
				$minlength = 0;
				$minstrict = false;	
			} else {
				$minlength = $arr['min'];
				$minstrict = $arr['minstrict'];
			}
			$caption   = $arr['caption'];
			$field     = $arr['field'];
			if ($field != '' && isset($row[$field])) {
				if ($minstrict && strlen($row[$field]) <= $minlength) {
					$result .= ($result != ''?',':'').'"'.$caption.'" '
						.mxl_cfg_exceptions::$dict_validator_rulescommon_checkPassword1.'> '
						.$minlength;
				} else if (!$minstrict && strlen($row[$field]) < $minlength) {
					$result .= ($result != ''?',':'').'"'.$caption.'" '
						.mxl_cfg_exceptions::$dict_validator_rulescommon_checkPassword1.'>= '
						.$minlength;
				} 
				if ($maxstrict && strlen($row[$field]) >= $maxlength) {
					$result .= ($result != ''?',':'').'"'.$caption.'" '
						.mxl_cfg_exceptions::$dict_validator_rulescommon_checkPassword1.'< '
						.$maxlength;
				} else if (!$maxstrict && strlen($row[$field]) > $maxlength) {
					$result .= ($result != ''?',':'').'"'.$caption.'" '
						.mxl_cfg_exceptions::$dict_validator_rulescommon_checkPassword1.'<= '
						.$maxlength;
				}
				if (preg_match('/['.mxl_cfg_exceptions::$dict_validator_rulescommon_checkPassword2.']|.\s/i', 
						$row[$field])) {
					$result .= ($result != ''?',':'').'"'.$caption.'" '
						.mxl_cfg_exceptions::$dict_validator_rulescommon_checkPassword3;
				}
			}
		}
		if ($result != '')
			$result = '<br>'.mxl_cfg_messages::$dict_validator_rulescommon_checkPassword
					.':<br>'.$result.'.';
		else
			$result = true;
		return $result;
	}
	
	/**
	 * Метод валидации по регулярному выражению.
	 * Возвращает true в случае успешной проверки, иначе сообщение о нарушении правила.
	 * Управляющие символы, используемые в регулярном выражении, должны быть 
	 * экранированы. Экранируются: одиночная кавычка ('), двойная кавычка ("), 
	 * обратный слэш (\) и NULL(байт NULL).
	 * 
	 * @param string $rule - параметр для передачи правила проверки, проверяемые
	 * 		поля передаются через ';'. В правиле через ':' указывается имя поля и 
	 * 		регулярное выражение. Если необходимо указать заголовок поля, который 
	 * 		будет отображаться при выводе сообщений по этому полю, то параметр 
	 * 		задается через ':' после имени поля и перед регулярным выражением. 
	 * 		Если необходимо указать сообщение, которое будет возвращаться в случае 
	 * 		неудачной проверки, то параметр задается через ':!' после регулярного 
	 * 		выражения. Чтобы вставить в сообщение ссылку на поле, в нужном месте 
	 * 		сообщения включите %f% (см. пример)
	 * 		пример: 'field1:caption1:match1;field2:match2;field3:match3:!Сообщение для поля %f%.;...' 
	 * @param array $row - массив данных для валидации по регулярному выражению 
	 * @return boolean | string
	 */
	public static function match($rule, $row){
		$fieldsToCheck = explode(';', $rule);
		$result = '';
		$result1 = '';
		$result2 = '';
		foreach ($fieldsToCheck as $field) {
			$caption = '';
			$match   = '';
			$message = '';
			if (strpos($field, ':') !== false){
				$x = explode(':', $field);
				if (count($x) == 4) {
					$field   = trim($x[0]);
					$caption = trim($x[1]);
					$match   = trim($x[2]);
					$message = substr(trim($x[3]), 1, strlen($x[3]));
					unset($x);	
				} else if (count($x) == 3) {
					$field   = trim($x[0]);
					if (strpos(trim($x[2]), '!') == 0) {
						$caption = trim($field);
						$match   = trim($x[1]);
						$message = substr(trim($x[2]), 1, strlen($x[2]));
					} else {
						$caption = trim($x[1]);
						$match   = trim($x[2]);
					}					
					unset($x);
				} else if (count($x) == 2) {
					$field   = trim($x[0]);
					$caption = trim($field);
					$match   = trim($x[1]);
					unset($x);	
				} 
			}
			if ($field != '' && isset($row[$field])) {
				if (!preg_match($match, $row[$field])) {
					if ($message != '') {
						$result1 .= ($result1 != ''?'<br>':'')
							.str_replace("%f%", '"'.$caption.'" ', $message);
						$message = '';
					} else {
						$result2 .= ($result2 != ''?',':'').'"'.$caption.'" '
							.mxl_cfg_exceptions::$dict_validator_rulescommon_match
							.$match;	
					}
				}
			}
		}
		if ($result1 != '')
			$result .= '<br>'.$result1;
		if ($result2 != '')
			$result .= '<br>'.mxl_cfg_messages::$dict_validator_rulescommon_match
					.':<br>'.$result2.'.';
		if ($result == '')
			$result = true;
		return $result;
	}
	
	/**
	 * Вспомогательный метод для проверки соответствия значения поля его типу и 
	 * парметрам (длина строки, точность вещественных типов, формат даты). 
	 * Возвращает true в случае успешной проверки, иначе сообщение о нарушении правила.
	 * По умолчанию, если не указан параметр, не проверяется длина строки и точность 
	 * вещественых типов.
	 * Формат по умолчанию для типа date - 'd/m/Y', для типа datetime - 'Y-m-d H:i:s'.
	 * Внимание: Валидация типа date и datetime нужна в случае, если не используется 
	 * визуальный компонент "дата-время"
	 * 
	 * @param string $field - имя поля
	 * @param string $value - значение поля
	 * @param string $type  - тип поля
	 * @param string|int $prop - параметр для свойства поля:
	 * 		для string - длина строки,
	 * 		для double, float - длина вещественной части,
	 * 		для date, datetime - формат даты
	 * @return boolean | string
	 */
	private static function typeIsValid($field, $value, $type, $prop) {
		$result = true;
		switch ($type) {
			case 'int':
				$result = true; 
				if (!preg_match('/^[0-9]+$/', $value)) $result = false;
				break;
			case 'boolean':
				$result = true; 
				if (!preg_match('/(^true$)|(^false$)/', $value)) $result = false;
				break;
			case 'double' | 'float':
				$lnum     = '[0-9]+';
				$dnum     = '([0-9]*[\.]'.$lnum.'$)|(^'.$lnum.'[\.][0-9]*)';
				$exp_dnum = '([+-]?(('.$lnum.'|'.$dnum.')[eE][+-]?'.$lnum.'))'; 
				$pattern  = '/(^'.$lnum.'$)|(^'.$dnum.'$)|(^'.$exp_dnum.'$)/';
				$result = true;
				if (preg_match($pattern, $value)) {
					if (preg_match('/^[0-9]+$/', $prop)) {
						$x = explode('.', $value);
						if (count($x) == 1) $buf = $x[0];
						else if (count($x) == 2) $buf = $x[1];
						if (stripos($buf, 'e') !== false) {
							$buf = substr($buf, 0, stripos($buf, 'e'));
						} 
						if (strlen($buf) > $prop) 
							$result = ' ('.mxl_cfg_exceptions::$dict_validator_rulescommon_checkType1
								.' {*}.{'.$prop.'})';
					}
				} else {
					$result = '';
				}
				break;
			case 'date':
				$prop = $prop == '' ? 'd/m/Y' : $prop;
				$arr = date_parse_from_format($prop, $value);
				if ($arr['warning_count'] != 0 || $arr['error_count'] != 0)
					$result = ' ('.mxl_cfg_exceptions::$dict_validator_rulescommon_checkType2
						.$prop.')';
				else
					$result = true;
				unset($arr);
				break;
			case 'datetime':
				$prop = $prop == '' ? 'Y-m-d H:i:s' : $prop;
				$arr = date_parse_from_format($prop, $value);
				if ($arr['warning_count'] != 0 || $arr['error_count'] != 0)
					$result = ' ('.mxl_cfg_exceptions::$dict_validator_rulescommon_checkType2
						.$prop.')';
				else
					$result = true;
				unset($arr);
				break;
			case 'string':
				$result = true;
				if (is_string($value)) {
					if (preg_match('/^[0-9]+$/', $prop) && strlen($value) > $prop) {
						$result = ' ('.mxl_cfg_exceptions::$dict_validator_rulescommon_checkType3
							.$prop.')';
					}
				} else {
					echo 'Не строка!';
					$result = '';
				} 
				break;
		}
		return $result;		
	}
	
	/**
	 * Метод проверки типа данных и параметров полей. Возвращает true в случае 
	 * успешной проверки, иначе сообщение о нарушении правила.
	 *  
	 * @param string $rule - параметр для передачи правила проверки типа.
	 * 		Правила для проверки передаются через ';'. В правиле через ':'
	 * 		указывается имя поля и тип поля. Если необходимо указать заголовок 
	 * 		поля, который будет отображаться при выводе сообщений по этому полю,
	 * 		то в параметр задается через ':' после имени поля и перед типом поля.
	 *		Тип поля может принимать следующие значения: int, boolean, float,
	 *		double, date, datetime, string.
	 *		Дополнительные параметры, такие как: длина строки, длина вещественной 
	 *		части, формат даты, указываются в круглых скобках рядом с типом поля.
	 *		По умолчанию длина строки и вещественной части чисел с плавающей 
	 *		точкой не имеют ограничений, формат для типа date - 'd/m/Y', 
	 *		для datetime - 'Y-m-d H:i:s'. 
	 * 		пример: 'field1:int;field2:caption2:string(50);field3:double(3);field4:date(d-m-Y);...' 
	 * @param array $row - массив для проверки типа полей
	 * @return boolean | string
	 */
	public static function checkType($rule, $row){
		$result = '';
		$fieldsToCheck = explode(';', $rule);
		foreach ($fieldsToCheck as $field){
			$caption = '';
			$type    = '';
			$prop    = '';
			if (strpos($field, ':') !== false) {
				$x = explode(':', $field);
				if (count($x) == 2) {
					$caption = $field = trim($x[0]);
					$type    = trim($x[1]);
				} else if (count($x) == 3) {
					$field   = trim($x[0]);
					$caption = trim($x[1]);
					$type    = trim($x[2]);
				}
				unset($x);
				if (strpos($type, '(') !== false) {
					$x = explode('(', $type);
					$type = trim($x[0]);
					$prop = substr($x[1], 0, strlen($x[1]) - 1);
				}
			}
			if ($field != '' && isset($row[$field])) {
				$err = mxl_dict_validator_rulescommon::typeIsValid($field, $row[$field], $type, $prop);
				if ($err !== true) {
					$result .= ($result != ''?',':'').'"'
						.$caption.'" : "'.$type.'"'.$err;	
				}	
			}	
		}
		if ($result != '')
			$result = '<br>'.mxl_cfg_messages::$dict_validator_rulescommon_checkType
					.':<br>'.$result.'.';
		else
			$result = true;
		return $result;
	}
}
?>
