<?php
/**
 * Файл для описания класса визуального компонента "Список", 
 * {@link mxl_widget_list}.
 * @package mxl_widget
 * @author merlin <merlin@ersys.ru>
 * @copyright Copyright (c) 2010, ООО НПО Метарекс
 */
/**
 * Класс описания виждета списка.
 * 
 * Пример использования:
 * <code>
 * 
 * </code>
 * 
 * @package mxl_widget
 * @author merlin <merlin@ersys.ru>
 * @copyright Copyright (c) 2010, ООО НПО Метарекс, {@link http://www.metarex.ru www.metarex.ru}
 */
class mxl_widget_list extends mxl_widget_widget{
	/**
	 * Определяет имя файла контролера, которому будут посылаться запросы. 
	 */
	const fileNameRequestPhpFile = 'getlist.php';

	/**
	 * Имя комбобокса на основании, которого строится виджет. Имя служит для 
	 * получения данных от модели списка.
	 * @var string
	 */
	private $listName;
	
	/**
	 * Модель комбобокса. Определяется в конструкторе по {@link $listName}
	 * @var mxl_dict_list
	 */
	private $linkCFG;
	
	/**
	 * Определяет будет ли загрузка происходить автоматически или вручную
	 * @var boolean 
	 */
	private $autoload;
	
	/**
	 * Конструктор класса виджета "Список".
	 * @param string $name - имя виджета (должно совпадать с именем id)
	 * @param string $listName - имя модели списка
	 * @param string $params - параметры
	 * @param boolean $autoload - Определяет будет ли загрузка происходить
	 * 		автоматически или вручную
	 * @return mxl_widget_list
	 */
	function __construct($name, $listName, $params = '', $autoload = true){
		parent::__construct($name);
		$this->listName  = $listName;
		$this->autoload   = $autoload;
		
		$MDS = mxl_core_metadatas::instance();
		if ($params != '')
			$MDS->setCurrParams($params);
		else
			$MDS->setCurrParamsToEmpty();
		$this->setParams($MDS->getCurrParams());

		$this->linkCFG = $MDS->getMetaData(mxl_core_metadatas::$list, 
				$listName);
	}
	
	/**
	 * Функция для получения имени модели списка
	 * @return string
	 */
	public function getListName(){
		return $this->listName;
	} 
	
	/**
	 * Функция для получения объекта модели списка.
	 * @return mxl_dict_list
	 */
	public function getLinkCFG(){
		return $this->linkCFG;
	} 
	
	/**
	 * Функция для определения полного url адресса для посылки запросов
	 * от клиентсокй части к серверной. 
	 * @return string
	 */
	public function getUrlRequestPhpFile(){
		$url = mxl_cfg_url::$metaxlib.'widget'.mxl_cfg_url::$separator
				.'request'.mxl_cfg_url::$separator
				.mxl_widget_list::fileNameRequestPhpFile;
		return $url;
	}
	
	/**
	 * Функция для получения ajax кода инициализации и работы компонента.
	 * Генерируемый код делится на несколько частей, которые будут размещены в 
	 * определенном порядке после генерации всего ajax кода.
	 * 
	 * @param string $declarePartCommon - декларативная общая часть, размещается 1-ой.
	 * @param string $declarePartPers - декларативная часть для компонента, размещается 2-ой
	 * @param string $runPartFirst - часть которая будет выполнена 1-ой, размещается 3-ей, 
	 * 		но 1-ой в части выполнения startAutoGenScript
	 * @param string $runPartLast - часть которая будет выполнена последней, размещается 4-ей, 
	 * 		но 2-ой в части выполнения startAutoGenScript
	 * @param string $funcsAfterInit - функции которые будут выполнены поле инициализации,
	 * 		размещается 4-ой.
	 * @param boolean $append_SID - параметр (костыль) определяющий передавать ли $_SID через
	 * 		baseParam и необходимо ли определять функцию для получения _SID
	 */
	public function getJSCode(&$declarePartCommon, &$declarePartPers, 
			&$runPartFirst, &$runPartLast, &$funcsAfterInit, $append_SID = false){
		global $_SID;
		
		$MDS = mxl_core_metadatas::instance();
		
		$nameWG   = $this->getName();
		$nameCFG  = $this->getListName();
		$LST	  = $this->getLinkCFG();
		
		$url      = $this->getUrlRequestPhpFile();
		
		$nm1 = 'store_lst_'.$nameWG;
		$nm2 = 'frm_lst_'.$nameWG;
		
		$x = ''; 
		$y = '';
		
		$bparam  = '{listName: "'.$nameCFG.'"'; 
		if (! $MDS->getCurrParams()->isEmpty())
			$bparam .= ', params:'.$MDS->getCurrParams()->saveToJSON();
		$bparam .= ($append_SID ? ', sid:"'. $_SID.'"'     : '');
		$bparam .= '}';

		$y .= 'var '. $nm1.';'.NL;
		$x .= mxl_widget_extjs::get_list_data_Store($nm1, $LST, $bparam, $url, 
				$this->autoload);
		if ($this->autoload)
			$x .= $nm1.'.load();'.NL;

		$y .= 'var '. $nm2.';'.NL;
		if ($LST->getKind() == mxl_dict_list::$lkMultiselect)
			$x .= mxl_widget_extjs::get_form_List_Multiselect($nameWG, $nm1, $nm2, $LST);
		elseif ($LST->getKind() == mxl_dict_list::$lkItemselector){
			$y .= 'var '. $nm1.'_res;'.NL;
			$x .= mxl_widget_extjs::get_form_List_Itemselector($nameWG, $nm1, $nm2, $LST);
		}
		

		$declarePartPers  = $y;
		$runPartFirst = $x;
	} 
	
	/**
	 * Функция проверки типа обработчика события визуального компонента.
	 * В случае удачной проверки возвращает true;
	 * @param string $type - проверяемый тип
	 * @return boolean
	 */
	public function checkEventHandlerType($type){
		return  false;		
	}
}
?>