<?php
/**
 * Файл конролеер для гридов - разбирает запрос сервера и в 
 * зависимости от вида запроса формирует ответ.
 * Обрабатываются запросы:<br>
 * 'get' - на получение данных;<br>
 * 'update' - на обновление значения поля в визуальной части;<br>
 * 'saverow' - на сохранение строки грида в БД и возвращение значения первичного
 * 		ключа;<br>
 * 'saverows' - сохранение массива измененных строк в БД, и возвращение массива
 * 		значений для поля первичного ключа в гриде;<br>
 * 'delete' - удаление сроки по значению первичного ключа;<br>
 * 'update_link_fields' - обновление значений связанных полей в визуальной части
 * 		грида.
 * 
 * В запросе обязательно должен присутствоавать параметр gridName - 
 * определяющий имя грида для которого необходимо обработать запрос.
 * 
 * В случае успешного запроса, ответ формируется функцией в зависимости от вида запроса:
 * <br>
 * 'get' - {@link mxl_dict_dbgrid::getJSON()};<br>
 * 'update' - {@link mxl_dict_dbgrid::updateField()};<br>
 * 'saverow' - {@link mxl_dict_dbgrid::saveRow()};<br>
 * 'saverows' - {@link mxl_dict_dbgrid::saveRows()};<br>
 * 'delete' - {@link mxl_dict_dbgrid::deleteRow()};<br>
 * 'update_link_fields' - {@link mxl_dict_dbgrid::updateLinkFields()}.
 * 
 * В случае неудачного запроса выводится {failure:true}.
 * 
 * @package mxl_widget
 * @author merlin <merlin@ersys.ru>
 * @copyright Copyright (c) 2009, ООО НПО Метарекс
 */
include_once realpath(dirname(__FILE__).'/../../init.php');

try {
	if (isset(mxl_cfg_url::$controller)){
		
		$postArr = array('q_controller_type'=>'grid');
		foreach ($_POST as $pi => $pv)
			$postArr[$pi] = $pv;
		if (!isset($_POST['action']))
			$postArr['action'] = 'get';
		
		$opts = array('http' =>
    		array(
        		'method'  => 'POST',
        		'header'  => 'Content-type: application/x-www-form-urlencoded',
		        'content' => http_build_query($postArr)
    		)
		);

		$context  = stream_context_create($opts);

		$result = file_get_contents(mxl_cfg_url::$controller, false, $context);	
        echo $result;
        
	} else {
	
		$MDS = mxl_core_metadatas::instance();

		$gridName = '';
		if (isset($_POST['gridName']))
			$gridName = $_POST['gridName'];

		$wc = '';
		if (isset($_POST['wc']))
			$wc = $_POST['wc'];
	
		$key = '';
		if (isset($_POST['key']))
			$key = $_POST['key'];

		$action = 'get';
		if (isset($_POST['action']))
			$action = $_POST['action'];
	
		
		$params = '';
		if (isset($_POST['params'])) {
			$params = $_POST['params'];
	
			$paramsMXL = new mxl_core_params();
			$paramsMXL->loadFromJSON($params);

			$MDS->setCurrParams($paramsMXL);
		}

		$dbg = $MDS->getMetaData(mxl_core_metadatas::$dbgrid, $gridName);

		if (isset($_POST['timeShift']))
			$dbg->setTimeShift($_POST['timeShift']);
	
		if (isset($_POST['table_name_suf']))
			$dbg->addTableNameSuf($_POST['table_name_suf']);

		if ($wc != '')
			$dbg->setWC($wc);

		switch($action){
    		case 'get':
				$start = $limit =  0;
				$sort  = $dir   = '';
				if ($dbg->isPagingMode()){ 
					if (isset($_POST['start'])) $start = $_POST['start'];
					if (isset($_POST['limit'])) $limit = $_POST['limit'];
					if (isset($_POST['sort']))  $sort  = $_POST['sort'];
					if (isset($_POST['dir']))   $dir   = $_POST['dir'];
				}
        		$dbg->getJSON($start, $limit, $sort, $dir);
		        break;
    	    
	    	case 'update':
    			$field = '';
				if (isset($_POST['field']))
					$field = $_POST['field'];
		
				$value = '';
				if (isset($_POST['value']))
					$value = $_POST['value'];
    	
	        	$dbg->updateField($key, $filed, $value);
		        break;
        
	    	case 'saverow':
    			$data = '{}';
				if (isset($_POST['data']))
					$data = $_POST['data'];
		
				$sucExt = '';
		    	if (isset($_POST['record_id']))
					$sucExt = 'record_id : "'.$_POST['record_id'].'"'; 
    		
    	
		        $dbg->saveRow($key, $data, $sucExt);
    		    break;
        
			case 'saverows':
    			$keys = '[]';
    			$datas = '[{}]';

		    	if (isset($_POST['keys']))
					$keys = $_POST['keys'];
    			if (isset($_POST['rowsData']))
					$datas = $_POST['rowsData'];
			
		        $dbg->saveRows($keys, $datas);
    		    break;
        	
	    	case 'delete':
	    	    $dbg->deleteRow($key);
    	    	break;
        
		    case 'search':
    		    break;
        
	    	case 'print':
    	    	break;
        
		    case 'update_link_fields':
				$value = '';
				if (isset($_POST['value']))
					$value = $_POST['value'];
		
				$fieldName = '';
				if (isset($_POST['fieldName']))
					$fieldName = $_POST['fieldName'];
			
				$dbg->updateLinkFields($fieldName, $key, $value);
	    		break;
    		
		    case 'export':
				$filename = '';
	    		if (isset($_POST['file'])) {
					$filename = $_POST['file'];
				}
				$format = '';
				if (isset($_POST['format'])) {
					$format = $_POST['format'];
				}
				$exporter = new mxl_utils_dbgridexport($dbg, $format, $filename);
	    		$exporter->getFile();
	    		break;
    	
    		default:
	        	echo '{failure:true}';
    	    	break;
		}
	}
} catch (Exception $e){
	echo '{failure:true, error:'.json_encode(mxl_utils_exceptions::toHtml($e)).'}';
}
?>
